/*
 * Copyright 2015,2016 SONY Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/list.h>
#include <linux/i2c.h>
#include <linux/irq.h>
#include <linux/jiffies.h>
#include <linux/uaccess.h>
#include <linux/delay.h>
#include <linux/interrupt.h>
#include <linux/io.h>
#include <linux/platform_device.h>
#include <linux/gpio.h>
#include <linux/miscdevice.h>
#include <linux/spinlock.h>
#include <linux/cdev.h>
#include <linux/of_gpio.h>
#include <linux/time.h> 
#include <linux/poll.h>

#include <media/v4l2-device.h>
#include <media/v4l2-ioctl.h>
#include <mach/mt_gpio.h>
#include <mach/eint.h>

#include <sound/core.h>
#include <sound/pcm.h>
#include <sound/pcm_params.h>
#include <sound/soc.h>
#include <sound/initval.h>

#include <linux/platform_data/ricoh5b_tuner.h>
#include "ricoh5b_cmd.h"
#include "ricoh5b_main.h"
#include "ricoh5b_fm_cmd.h"

#ifdef CONFIG_REGMON_DEBUG
#include <mach/regmon.h>
#endif

//#define  RICOH5B_MAIN_DEBUG  RICOH5B_CONF_DEBUG

/* Chip Power ON wait time 500ms */
#define CHIP_POWER_ON_WAIT_TIME    500000  /* us */

/* Block Power ON wait time
 * X'tal 32kHz -> 100ms
 * X'tal 12MHz -> 30ms  */
#define BLOCK_POWER_ON_WAIT_TIME    100000  /* us */

/* muting wait time 20ms */
#define MUTING_WAIT_TIME    20000    /* us */

#define	MTK_I2C_READ_WRITE_LEN(read_len, write_len) (((read_len) << 8) | ((write_len) << 0))

/* repeat state check */
#define RDS_FLAG                 1
#define REC_AUDIO_MODE_FLAG      2
#define SIGNAL_STRENGTH_FLAG     3
#define RICOH5B_STATE_POLL_TIME  (500/(1000/HZ)) //ms

/* ricoh5b init parameter */
#define RICOH5B_MUTE_ON              1
#define RICOH5B_MUTE_OFF             0
#define RICOH5B_MODE_STEREO          1
#define RICOH5B_MODE_MONO            0
#define RICOH5B_FIRST_FREQ           80000
#define RICOH5B_FIRST_SENSE_MODE     0x00
#define RICOH5B_FIRST_SENSE_HIGH     19
#define RICOH5B_FIRST_SENSE_LOW      19
#define RICOH5B_RDS_MODE_ON          1
#define RICOH5B_RDS_MODE_OFF         0
#define RICOH5B_SIGNAL_STRENGTH_MASK 0x7F

/* ricoh5b rds use parameter */
#define RICOH5B_RDS_NOERR            0x80
#define RICOH5B_RDS_UNC              0x40
#define RICOH5B_RDS_BLOCK            0x38
#define RICOH5B_RDS_BLOCK_B          0x01
#define RICOH5B_RDS_BLOCK_C          0x03
#define ACCESSARY_CHECK_DELAY_TIME   (1/(1000/HZ))    //1ms
/* SAKANA 20160419 KAMIJO [start] */
#if 0
	#define RICOH5B_RDSI_EINT            117  /* GPIO 205 */
#else
	#define RICOH5B_RDSI_EINT            16   /* GPIO  34 */
#endif
/* SAKANA 20160419 KAMIJO [end] */
#define RICOH5B_RDS_BLOCK_B_PS       0x00
#define RICOH5B_RDS_BUF          8
#define RICOH5B_RDS_BLOCK_MASK       0xF0
#define RICOH5B_RDS_SEGMENT_MASK     0x03
#define RICOH5B_RDS_SEGMENT_ADDR_1   0x00
#define RICOH5B_RDS_SEGMENT_ADDR_2   0x01
#define RICOH5B_RDS_SEGMENT_ADDR_3   0x02
#define RICOH5B_RDS_SEGMENT_ADDR_4   0x03
#define RICOH5B_RDS_LIMIT_MIN        0x20
#define RICOH5B_RDS_LIMIT_MAX        0xFE
#define RICOH5B_LAST_DATA            7
#define FIRST_POSITION               0
#define SECOND_POSITION              2
#define THIRD_POSITION               4
#define FOURTH_POSITION              6
#define RDS_WRITE_ENABLE             1
#define RDS_WRITE_DISABLE            0
#define RDS_BLOCK_SHIFT              3
#define RICOH5B_RDS_RETRY            5

/* ricoh5b use pin */
#define RICOH5B_I2S5_DATA_GPIO       16
#define RICOH5B_I2S5_BCK_GPIO        17
#define RICOH5B_I2S5_LRCK_GPIO       30

/* de-emphasis mode */
#define DEEMPHASIS_LOW               0x00
#define DEEMPHASIS_DEMP_HIGH         0x40
#define DEEMPHASIS_DEMP_CE_HIGH      0x01

#define RICOH5B_RSSI_THRESHOLD       19      /* dBu */
#define RICOH5B_I2C_RETRY_COUNT      5
#define SIGNAL_STRENGTH_RETRY        6

/* poll status */
#define POLL_CHECK_FLAG_ENABLE       1
#define POLL_CHECK_FLAG_DISABLE      0

/* Power On Sequence Step */
typedef enum {
	P_ON_STEP_REG_RESET,
	P_ON_STEP_MODULE_ON,
	P_ON_STEP_XTAL_SETTING,
	P_ON_STEP_SYNTHE_SETTING,
	P_ON_STEP_AGC_SETTING,
	P_ON_STEP_DSP_INIT_SETTING,
	P_ON_STEP_CHIP_ON,
	P_ON_STEP_WAIT_CHIP_ON,
	P_ON_STEP_BLOCK_ON,
	P_ON_STEP_WAIT_BLOCK_ON,
	P_ON_STEP_CK_GEN_SETTING,
	P_ON_STEP_RDSI_SETTING,
	P_ON_STEP_AUDIO_SETTING,
	P_ON_STEP_RTRIM_SETTING,
	P_ON_STEP_END
} P_ON_STEP;

/* Mute On Sequence Step */
typedef enum {
	MUTE_ON_STEP_SET_WAIT_COUNT,
	MUTE_ON_STEP_WAIT_WAIT_COUNT,
	MUTE_ON_STEP_MUTE_ON,
	MUTE_ON_STEP_WAIT_MUTE_ON,
	MUTE_ON_STEP_END
} MUTE_ON_STEP;

/* Mute Off Sequence Step */
typedef enum {
	MUTE_OFF_STEP_MUTE_OFF,
	MUTE_OFF_STEP_WAIT_MUTE_OFF,
	MUTE_OFF_STEP_SET_WAIT_COUNT,
	MUTE_OFF_STEP_END
} MUTE_OFF_STEP;

/* adjust AGCDET offset Sequence */
typedef enum {
	AGCDET_STEP_SET_AGCDET_OFFSET,
	AGCDET_STEP_GET_AD6,
	AGCDET_STEP_CALC_BEST_DET,
	AGCDET_STEP_JUDGE_DET,
	AGCDET_STEP_END
} AGCDET_OFF_STEP;

/* align AGCG Sequence */
typedef enum {
	AGCG_STEP_SET_PGM,
	AGCG_STEP_WAIT_PGM,
	AGCG_STEP_CALC_BEST_IML,
	AGCG_STEP_IML_ADJUST,
	AGCG_STEP_GET_IMLI,
	AGCG_STEP_JUDGE_IML,
	AGCG_STEP_SET_AVECOUNT_128,
	AGCG_STEP_IQ_CORRECT,
	AGCG_STEP_GET_PGM,
	AGCG_STEP_GET_PG,
	AGCG_STEP_PG_3_RETRIEVE,
	AGCG_STEP_PG_1_RETRIEVE,
	AGCG_STEP_PG_2_RETRIEVE,
	AGCG_STEP_PG_2_IQ_CORRECT,
	AGCG_STEP_PG_2_GET_PG,
	AGCG_STEP_PG_2_JUDGE_PG,
	AGCG_STEP_SET_AVECOUNT_16,
	AGCG_STEP_SET_END
} AGCG_OFF_STEP;

/* CTUNE Sequence */
typedef enum {
	CTUNE_STEP_START_CTUNE,
	CTUNE_STEP_GET_CTUNE,
	CTUNE_STEP_END
} CTUNE_STEP;

struct ricoh5b_dev {
	wait_queue_head_t read_wq;
	struct mutex read_mutex;
	struct mutex station_mutex;
	struct mutex rds_mode_mutex;
	struct mutex repeat_check;
	struct i2c_client *client;
	struct video_device *video_device;
	struct workqueue_struct * workqueue;
	struct delayed_work ricoh5b_rds_work;
	int audio_mode;
	int mute_state;
	int tuned_freq;
	int sense_mode;
	int threshold_val;
	int rds_mode;
	int deemp;
	int deemp_ce;
	int station_check;
	int rec_audio_mode;
	unsigned char sense_high;
	unsigned char sense_low;
	unsigned int ce_gpio;
	unsigned int iic_gpio;
	unsigned int vdd_gpio;
};

static struct workqueue_struct * workqueue_check = 0;
static struct delayed_work ricoh5b_check_work;
static char tuner_open = 0;
static char res_signal_strength    = 0;
static char change_signal_strength = 0;
static char change_audmode         = 0;
static char change_rds_data        = 0;
static int  gain_value  = RICOH5B_VOLUME_0DB;
static int  rbds_enable = RICOH5B_REG_RDS_ON_VALUE;

static int DeEmphasis = DEEMPHASIS_50US;
module_param_named(deemphasis, DeEmphasis, int, 0);
MODULE_PARM_DESC(deemphasis, "deemphasis");

static int GainValue = GAIN_0DB;
module_param_named(gainvalue, GainValue, int, 0);
MODULE_PARM_DESC(gainvalue, "gainvalue");

static int RdsMode = RDS_MODE_RDS;
module_param_named(rdsmode, RdsMode, int, 0);
MODULE_PARM_DESC(rdsmode, "rdsmode");

static struct ricoh5b_dev *ricoh5b_device_info;
static struct i2c_client *ricoh5b_i2c_client;

/* operation_routines */
static int ricoh5b_read(
	struct file *file,
	void *buf,
	size_t len
);

static int ricoh5b_open(
	struct file * file
);
static int ricoh5b_close(
	struct file * file
);
static int ricoh5b_poll(
	struct file * file,
	poll_table * wait
);
static int ricoh5b_querycap(
	struct file            * file,
	void                   * private,
	struct v4l2_capability * capability
);
static int ricoh5b_s_tuner(
	struct file       * file,
	void              * private,
	struct v4l2_tuner * tuner
);
static int ricoh5b_g_tuner(
	struct file       * file,
	void              * private,
	struct v4l2_tuner * tuner
);
static int ricoh5b_s_frequency(
	struct file           * file,
	void                  * private,
	struct v4l2_frequency * frequency
);
static int ricoh5b_g_frequency(
	struct file           * file,
	void                  * private,
	struct v4l2_frequency * frequency
);
static int ricoh5b_s_ctrl(
	struct file         * file,
	void                * private,
	struct v4l2_control * control
);
static int ricoh5b_g_ctrl(
	struct file         * file,
	void                * private,
	struct v4l2_control * control
);

static int ricoh5b_sense_mode_set(
    struct ricoh5b_dev *ricoh5b_dev,
    int value
);

static int ricoh5b_threshold_set(
    struct ricoh5b_dev *ricoh5b_dev,
    int value
);

static int ricoh5b_station_check(struct ricoh5b_dev *ricoh5b_dev);
static int ricoh5b_rssi_check(int sens, int *value);
static int ricoh5b_fo_sat_check(void);
static int ricoh5b_rec_audio_mode(void);
static void ricoh5b_next_rds_eint(struct ricoh5b_dev *ricoh5b_dev);
static int ricoh5b_read_reg(int address, int *value);

static const struct v4l2_file_operations ricoh5b_file_operations = {
	.owner           = THIS_MODULE,
	.open            = ricoh5b_open,
	.release         = ricoh5b_close,
	.read            = ricoh5b_read,
	.unlocked_ioctl  = video_ioctl2,
	.poll            = ricoh5b_poll,
};

static const struct v4l2_ioctl_ops ricoh5b_ioctl_operations = {
	.vidioc_querycap       = ricoh5b_querycap,
	.vidioc_s_tuner        = ricoh5b_s_tuner,
	.vidioc_g_tuner        = ricoh5b_g_tuner,
	.vidioc_s_frequency    = ricoh5b_s_frequency,
	.vidioc_g_frequency    = ricoh5b_g_frequency,
	.vidioc_s_ctrl         = ricoh5b_s_ctrl,
	.vidioc_g_ctrl         = ricoh5b_g_ctrl,
};

/*---------------*/
/* command table */
/*---------------*/

/* register reset */
static const RICOH5B_REG ricoh5b_cmd_reg_reset_tbl[] = {
    { RICOH5B_REG_ADR_SRST,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_SRST_ALL  },
    { RICOH5B_REG_ADR_MUTE,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_MUTE_ON   }
};

/* module power ON */
static const RICOH5B_REG ricoh5b_cmd_module_power_on_tbl[] = {
    { RICOH5B_REG_ADR_BLOCK2_PE,  RICOH5B_REG_BIT_BLOCK2_PE,  RICOH5B_REG_DAT_PELO_ON | RICOH5B_REG_DAT_PEFMMIX_ON |
      RICOH5B_REG_DAT_PEFMLNA_ON    },
    { RICOH5B_REG_ADR_VCO,        RICOH5B_REG_NON_MASK,       RICOH5B_REG_DAT_VCO                                    },
    { RICOH5B_REG_ADR_IMFMEN_EN,  RICOH5B_REG_NON_MASK,       RICOH5B_REG_DAT_IMFMEN_ON | RICOH5B_REG_DAT_FMEN_ON    },
    { RICOH5B_REG_ADR_SPOFS_SYN,  RICOH5B_REG_NON_MASK,       RICOH5B_REG_DAT_SPOFS_SYN                              }
};

/* X'tal setting */
static const RICOH5B_REG ricoh5b_cmd_xtal_setting_tbl[] = {
#if (RICOH5B_SELECT_X_TAL == RICOH5B_X_TAL_32KHZ)
    { RICOH5B_REG_ADR_XTAL,        RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_XTAL_32K                                            },
    { RICOH5B_REG_ADR_PELPFVGA,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_PELPFVGA_ON |
      RICOH5B_REG_DAT_MIX_ISEL_I_SOURCE },
    { RICOH5B_REG_ADR_CLK_WIN,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_CLK_WIN_32K                                         },
    { RICOH5B_REG_ADR_CK_VCO,      RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_CK_VCO_32K                                          },
    { RICOH5B_REG_ADR_VTJHA_CK,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_VTJHA_CK_0_96 | RICOH5B_REG_DAT_VTJLA_CK_0_24 |
      RICOH5B_REG_DAT_VTJLM_CK_0_28 },
    { RICOH5B_REG_ADR_CKBRD,       RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_CKBRD | RICOH5B_REG_DAT_CKBRS                       },
    { RICOH5B_REG_ADR_PADEN_CK,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_PADEN_CK_ENABLE | RICOH5B_REG_DAT_AMPEN_CK_ENABLE   },
    { RICOH5B_REG_ADR_SYN_VTJH,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_SYN_VTJH_0_92 | RICOH5B_REG_DAT_SYN_VTJL_0_20       },
    { RICOH5B_REG_ADR_LOC_T2,      RICOH5B_REG_BIT_LOC_T2,  RICOH5B_REG_DAT_LOC_T2_220U                                       },
    { RICOH5B_REG_ADR_RSW_CK,      RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_SW0_CK_32K | RICOH5B_REG_DAT_RSW_CK_5000K           },
    { RICOH5B_REG_ADR_CIO_SYN,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_CIO_SYN_2_25 | RICOH5B_REG_DAT_SYN_RDF_35K          },
#else
    { RICOH5B_REG_ADR_XTAL,        RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_XTAL_12M                                            },
    { RICOH5B_REG_ADR_PELPFVGA,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_XFS_12M | RICOH5B_REG_DAT_PELPFVGA_ON |
      RICOH5B_REG_DAT_MIX_ISEL_I_SOURCE },
    { RICOH5B_REG_ADR_CLK_WIN,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_CLK_WIN_12M                                         },
    { RICOH5B_REG_ADR_CK_VCO,      RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_CK_VCO_12M                                          },
    { RICOH5B_REG_ADR_PADEN_CK,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_PADEN_CK_DISABLE | RICOH5B_REG_DAT_AMPEN_CK_DISABLE },
    { RICOH5B_REG_ADR_LOC_T2,      RICOH5B_REG_BIT_LOC_T2,  RICOH5B_REG_DAT_LOC_T2_100U                                       },
    { RICOH5B_REG_ADR_RSW_CK,      RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_SW0_CK_12M | RICOH5B_REG_DAT_RSW_CK_760K            },
    { RICOH5B_REG_ADR_CIO_SYN,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_CIO_SYN_18_0 | RICOH5B_REG_DAT_SYN_RDF_100K         },
#endif
    { RICOH5B_REG_ADR_RSYS,        RICOH5B_REG_BIT_RSYS,  RICOH5B_REG_DAT_RSYS_FM400K_AMAUTO                                  },
    { RICOH5B_REG_ADR_ISEL2,       RICOH5B_REG_NON_MASK,  RICOH5B_REG_BIT_XBOOST_BOOST | RICOH5B_REG_DAT_ISEL2_NORMAL         },
    { RICOH5B_REG_ADR_LOSTART_EN,  RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_LOSTART_EN_ENABLE | RICOH5B_REG_DAT_LOSTART         }
};

/* synthe setting */
static const RICOH5B_REG ricoh5b_cmd_synthe_setting_tbl[] = {
    { RICOH5B_REG_ADR_ADCMOD,      RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_ADCMOD_EXCHANGE_IQ           },
#if RICOH5B_USE_TUNER
    { RICOH5B_REG_ADR_AD1_RSTB,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AD1_RSTB_NORMAL              },
#endif
    { RICOH5B_REG_ADR_BW,          RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_BW_DEFAULT                   }
};

/* AGC setting */
static const RICOH5B_REG ricoh5b_cmd_agc_setting_tbl[] = {
    { RICOH5B_REG_ADR_FMIN,          RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_FMIN_24DB | RICOH5B_REG_DAT_PGM_18DB       },
    { RICOH5B_REG_ADR_AGC_AUTO,      RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AGC_AUTO_OFF                               },
    { RICOH5B_REG_ADR_TABLE_MODE,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_TABLE_MODE                                 },
    { RICOH5B_REG_ADR_BLOCK1_PE,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_BLOCK1_AGC_SETTING                         },
    { RICOH5B_REG_ADR_AM_VIFO_UPPER,  RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_VIFO_UPPER                             },
    { RICOH5B_REG_ADR_AM_VIFO_LOWER,   RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_VIFO_LOWER                            },
    { RICOH5B_REG_ADR_AM_VUD_UPPER,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_VUD_UPPER                             },
    { RICOH5B_REG_ADR_AM_VUD_LOWER,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_VUD_LOWER                             },
    { RICOH5B_REG_ADR_AM_PIF_SCALE,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_PIF_SCALE                             },
#if RICOH5B_USE_TUNER
    { RICOH5B_REG_ADR_AM_THR_IFR,      RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_THR_IFR | RICOH5B_REG_DAT_AM_DEC_IFR  },
    { RICOH5B_REG_ADR_WIN_CNT_VAL,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_WIN_CNT_VAL                              },
    { RICOH5B_REG_ADR_AM_VIFO_INIT,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_VIFO_INIT                             },
    { RICOH5B_REG_ADR_AM_VUD_INIT,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_VUD_INIT                              },
    { RICOH5B_REG_ADR_AM_PIF_INIT,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_AM_PIF_INIT                              }
#endif
};

/* DSP initial setting */
static const RICOH5B_REG ricoh5b_cmd_dsp_init_setting_tbl[] = {
    { RICOH5B_REG_ADR_AM_DAGC,        RICOH5B_REG_NON_MASK,      RICOH5B_REG_DAT_AM_DAGC                            },
#if RICOH5B_USE_TUNER
    { RICOH5B_REG_ADR_AM_TH_DET,      RICOH5B_REG_NON_MASK,      RICOH5B_REG_DAT_AM_TH_DET                          },
#endif
    { RICOH5B_REG_ADR_FM_RSSI_CONST,  RICOH5B_REG_NON_MASK,      RICOH5B_REG_DAT_FM_RSSI_CONST                      },
    { RICOH5B_REG_ADR_SMT,            RICOH5B_REG_NON_MASK,      RICOH5B_REG_DAT_SMT_OFF                            },
    { RICOH5B_REG_ADR_HCC,            RICOH5B_REG_NON_MASK,      RICOH5B_REG_DAT_HCC_OFF | RICOH5B_REG_DAT_SNC_OFF  },
    { RICOH5B_REG_ADR_VBFSEL,         RICOH5B_REG_NON_MASK,      RICOH5B_REG_DAT_VBFSEL_DEFAULT                     },
    { RICOH5B_REG_ADR_DET_DC_ADJ,     RICOH5B_REG_NON_MASK,      RICOH5B_REG_DAT_DET_DC_ADJ_PLUS_0                  },
    { RICOH5B_REG_ADR_ITRIM,          RICOH5B_REG_NON_MASK,      RICOH5B_REG_DAT_ITRIM_6MA                          },
    { RICOH5B_REG_ADR_COM_SEL,        RICOH5B_REG_BIT_COM_SEL,   RICOH5B_REG_DAT_COM_SEL_300M                       },
    { RICOH5B_REG_ADR_AD6_I_SW,       RICOH5B_REG_BIT_AD6_I_SW,  RICOH5B_REG_DAT_AD6_I_SW                           }
};

/* Clock Generator setting */
static const RICOH5B_REG ricoh5b_cmd_clk_setting_tbl[] = {
#if (RICOH5B_SELECT_X_TAL == RICOH5B_X_TAL_32KHZ)
    { RICOH5B_REG_ADR_PADEN_CK,    RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_PADEN_CK_ENABLE
                                 | RICOH5B_REG_DAT_AMPEN_CK_ENABLE
                                 | RICOH5B_REG_DAT_CAPOUT_CK_ENABLE
                                 | RICOH5B_REG_DAT_CAPADJ_CK_0_25                },
    { RICOH5B_REG_ADR_RSW_CK,      RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_SW0_CK_32K | RICOH5B_REG_DAT_RSW_CK_400K  },
#endif
    { RICOH5B_REG_ADR_UL_MASK,     RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_UL_MASK_OFF     }
};

/* RDSI output setting */
static const RICOH5B_REG ricoh5b_cmd_rdsi_output_tbl[] = {
    { RICOH5B_REG_ADR_RDS_OUT_EN_H,  RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_RDS_OUT_EN_H  },
    { RICOH5B_REG_ADR_RDS_OUT_EN_L,  RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_RDS_OUT_EN_L  }
};

/* DSP alignment start */
static const RICOH5B_REG ricoh5b_cmd_dsp_align_start_tbl[] = {
    { RICOH5B_REG_ADR_BW,          RICOH5B_REG_BIT_BW,           RICOH5B_REG_DAT_BW_FM_75K                                    },
    { RICOH5B_REG_ADR_STATION_EN,  RICOH5B_REG_BIT_STATION_EN,   RICOH5B_REG_DAT_STATION_EN_ENABLE                            },
    { RICOH5B_REG_ADR_FO_SAT_SEL,  RICOH5B_REG_NON_MASK,         RICOH5B_REG_DAT_FO_SAT_10K | RICOH5B_REG_DAT_DCO_AVESEL_IDLE },
    { RICOH5B_REG_ADR_FC_SW,       RICOH5B_REG_BIT_FC_SW,        RICOH5B_REG_DAT_FC_SW_500K                                   },
    { RICOH5B_REG_ADR_IMFMSW,      RICOH5B_REG_BIT_IMFMSW | RICOH5B_REG_BIT_IMFMA,    RICOH5B_REG_DAT_IMFMSW_ENABLE |
      RICOH5B_REG_DAT_IMFMA_MIN },
    { RICOH5B_REG_ADR_AVECOUNT,    RICOH5B_REG_BIT_AVECOUNT,     RICOH5B_REG_DAT_AVECOUNT_16                                  },
    { RICOH5B_REG_ADR_FMIN,        RICOH5B_REG_NON_MASK,         RICOH5B_REG_DAT_FMIN_MINUS_36DB | RICOH5B_REG_DAT_PGM_0DB    },
    { RICOH5B_REG_ADR_XDOWN,       RICOH5B_REG_NON_MASK,         RICOH5B_REG_DAT_XDOWN_MUCH | RICOH5B_REG_DAT_DET_AFM_FM      }
};

/* DSP alignment end */
static const RICOH5B_REG ricoh5b_cmd_dsp_align_end_tbl[] = {
    { RICOH5B_REG_ADR_IMFMSW,      RICOH5B_REG_BIT_IMFMSW | RICOH5B_REG_BIT_IMFMA,    RICOH5B_REG_DAT_IMFMSW_DISABLE |
      RICOH5B_REG_DAT_IMFMA_MIN },
    { RICOH5B_REG_ADR_BW,          RICOH5B_REG_BIT_BW,                  RICOH5B_REG_DAT_BW_FM_130K                    },
    { RICOH5B_REG_ADR_FO_SAT_SEL,  RICOH5B_REG_NON_MASK,                RICOH5B_REG_DAT_FO_SAT_25K                    },
    { RICOH5B_REG_ADR_FC_SW,       RICOH5B_REG_BIT_FC_SW,               RICOH5B_REG_DAT_FC_SW_1M                      },
    { RICOH5B_REG_ADR_IMF,         RICOH5B_REG_NON_MASK,                RICOH5B_REG_DAT_IMF_OFF                       },
    { RICOH5B_REG_ADR_PGM,         RICOH5B_REG_BIT_PGM,                 RICOH5B_REG_DAT_PGM_0DB                       }
};

/* reset RFAGC */
static const RICOH5B_REG ricoh5b_cmd_reset_rfagc_tbl[] = {
    { RICOH5B_REG_ADR_RST_RFAGC,  RICOH5B_REG_BIT_RST_RFAGC,  RICOH5B_REG_DAT_RST_RFAGC_ENABLE   },
    { RICOH5B_REG_ADR_RST_RFAGC,  RICOH5B_REG_BIT_RST_RFAGC,  RICOH5B_REG_DAT_RST_RFAGC_DISABLE  }
};

/* Image Signal Output Level OFF */
static const RICOH5B_REG ricoh5b_cmd_im_output_off_tbl[] = {
    { RICOH5B_REG_ADR_IMFMA,  RICOH5B_REG_BIT_IMFMA,  RICOH5B_REG_DAT_IMFMA_MIN  }
};

/* Analog audio output */
static const RICOH5B_REG ricoh5b_cmd_audio_output_analog_tbl[] = {
    { RICOH5B_REG_ADR_IISD_SETTING,    RICOH5B_REG_NON_MASK,           RICOH5B_REG_DAT_IISD_SETTING_ANALOG    },
    { RICOH5B_REG_ADR_AUDIO_SETTING,   RICOH5B_REG_BIT_AUDIO_SETTING,  RICOH5B_REG_DAT_AUDIO_SETTING_ANALOG   },
    { RICOH5B_REG_ADR_IIS_EN,          RICOH5B_REG_BIT_IIS_EN,         RICOH5B_REG_DAT_IIS_EN_DAC_OUT         },
};

/* IIS 32fs audio output */
static const RICOH5B_REG ricoh5b_cmd_audio_output_iis_32fs_tbl[] = {
    { RICOH5B_REG_ADR_IISD_SETTING,    RICOH5B_REG_NON_MASK,            RICOH5B_REG_DAT_IISD_SETTING_DIGITAL                 },
    { RICOH5B_REG_ADR_AUDIO_SETTING,   RICOH5B_REG_BIT_AUDIO_SETTING,   RICOH5B_REG_DAT_AUDIO_SETTING_DIGITAL                },
    { RICOH5B_REG_ADR_IIS_EN,          RICOH5B_REG_BIT_IIS_SEL  |  RICOH5B_REG_BIT_IIS_32_64  |  RICOH5B_REG_BIT_IIS_EN,
      RICOH5B_REG_DAT_IIS_SEL_IIS_FORMAT  | RICOH5B_REG_DAT_IIS_32FS  |  RICOH5B_REG_DAT_IIS_EN_IIS_OUT }
};

/* IIS 64fs audio output */
static const RICOH5B_REG ricoh5b_cmd_audio_output_iis_64fs_tbl[] = {
    { RICOH5B_REG_ADR_IISD_SETTING,    RICOH5B_REG_NON_MASK,           RICOH5B_REG_DAT_IISD_SETTING_DIGITAL                 },
    { RICOH5B_REG_ADR_AUDIO_SETTING,   RICOH5B_REG_BIT_AUDIO_SETTING,  RICOH5B_REG_DAT_AUDIO_SETTING_DIGITAL                },
    { RICOH5B_REG_ADR_IIS_EN,          RICOH5B_REG_BIT_IIS_SEL  |  RICOH5B_REG_BIT_IIS_32_64  |  RICOH5B_REG_BIT_IIS_EN,
      RICOH5B_REG_DAT_IIS_SEL_IIS_FORMAT  | RICOH5B_REG_DAT_IIS_64FS  |  RICOH5B_REG_DAT_IIS_EN_IIS_OUT }
};

/* chip power off */
static const RICOH5B_REG ricoh5b_cmd_chip_power_off_tbl[] = {
    { RICOH5B_REG_ADR_BLOCK1_PE,  RICOH5B_REG_NON_MASK,  RICOH5B_REG_DAT_BLOCK1_POWER_OFF  },
    { RICOH5B_REG_ADR_PE,         RICOH5B_REG_BIT_PE,    RICOH5B_REG_DAT_PE_OFF            }
};

#define RICOH5B_CMD_REG_RESET_TBL_SIZE              (sizeof(ricoh5b_cmd_reg_reset_tbl)/sizeof(ricoh5b_cmd_reg_reset_tbl[0]))
#define RICOH5B_CMD_MODULE_POWER_ON_TBL_SIZE        (sizeof(ricoh5b_cmd_module_power_on_tbl)/sizeof(ricoh5b_cmd_module_power_on_tbl[0]))
#define RICOH5B_CMD_XTAL_SETTING_TBL_SIZE           (sizeof(ricoh5b_cmd_xtal_setting_tbl)/sizeof(ricoh5b_cmd_xtal_setting_tbl[0]))
#define RICOH5B_CMD_SYNTHE_SETTING_TBL_SIZE         (sizeof(ricoh5b_cmd_synthe_setting_tbl)/sizeof(ricoh5b_cmd_synthe_setting_tbl[0]))
#define RICOH5B_CMD_AGC_SETTING_TBL_SIZE            (sizeof(ricoh5b_cmd_agc_setting_tbl)/sizeof(ricoh5b_cmd_agc_setting_tbl[0]))
#define RICOH5B_CMD_DSP_INIT_SETTING_TBL_SIZE       (sizeof(ricoh5b_cmd_dsp_init_setting_tbl)/sizeof(ricoh5b_cmd_dsp_init_setting_tbl[0]))
#define RICOH5B_CMD_CLK_SETTING_TBL_SIZE            (sizeof(ricoh5b_cmd_clk_setting_tbl)/sizeof(ricoh5b_cmd_clk_setting_tbl[0]))
#define RICOH5B_CMD_RDSI_OUTPUT_TBL_SIZE            (sizeof(ricoh5b_cmd_rdsi_output_tbl)/sizeof(ricoh5b_cmd_rdsi_output_tbl[0]))
#define RICOH5B_CMD_DSP_ALIGN_START_TBL_SIZE        (sizeof(ricoh5b_cmd_dsp_align_start_tbl)/sizeof(ricoh5b_cmd_dsp_align_start_tbl[0]))
#define RICOH5B_CMD_DSP_ALIGN_END_TBL_SIZE          (sizeof(ricoh5b_cmd_dsp_align_end_tbl)/sizeof(ricoh5b_cmd_dsp_align_end_tbl[0]))
#define RICOH5B_CMD_RESET_RFAGC_TBL_SIZE            (sizeof(ricoh5b_cmd_reset_rfagc_tbl)/sizeof(ricoh5b_cmd_reset_rfagc_tbl[0]))
#define RICOH5B_CMD_IM_OUTPUT_OFF_TBL_SIZE          (sizeof(ricoh5b_cmd_im_output_off_tbl)/sizeof(ricoh5b_cmd_im_output_off_tbl[0]))
#define RICOH5B_CMD_AUDIO_OUTPUT_ANALOG_TBL_SIZE    (sizeof(ricoh5b_cmd_audio_output_analog_tbl)/sizeof(ricoh5b_cmd_audio_output_analog_tbl[0]))
#define RICOH5B_CMD_AUDIO_OUTPUT_IIS_32FS_TBL_SIZE  (sizeof(ricoh5b_cmd_audio_output_iis_32fs_tbl)/sizeof(ricoh5b_cmd_audio_output_iis_32fs_tbl[0]))
#define RICOH5B_CMD_AUDIO_OUTPUT_IIS_64FS_TBL_SIZE  (sizeof(ricoh5b_cmd_audio_output_iis_64fs_tbl)/sizeof(ricoh5b_cmd_audio_output_iis_64fs_tbl[0]))
#define RICOH5B_CMD_CHIP_POWER_OFF_TBL_SIZE         (sizeof(ricoh5b_cmd_chip_power_off_tbl)/sizeof(ricoh5b_cmd_chip_power_off_tbl[0]))

/*-------*/
/* other */
/*-------*/

/* align AGCG table */
static unsigned char  align_agcg_tbl[] = {
    RICOH5B_REG_DAT_PGM_0DB,
    RICOH5B_REG_DAT_PGM_6DB,
    RICOH5B_REG_DAT_PGM_12DB,
    RICOH5B_REG_DAT_PGM_18DB,
    RICOH5B_REG_DAT_PGM_24DB
};
#define ALIGN_AGCG_TBL_SIZE      (sizeof(align_agcg_tbl)/sizeof(align_agcg_tbl[0]))

/******************************************************************************/
/* global variables definition which are refered only this file               */
/******************************************************************************/
static RICOH5B_PG*          p_fine_pg;
static unsigned char      p_fine_pg_step = 0;
static unsigned char      det_offset = 0;

/********************/
/*@ regmon_routines */
/********************/

#ifdef CONFIG_REGMON_DEBUG

static int ricoh5b_regmon_write_reg(
	void         * private_data,
	unsigned int   address,
	unsigned int   value
);

static int ricoh5b_regmon_read_reg(
	void         * private_data,
	unsigned int   address,
	unsigned int * value
);

static regmon_reg_info_t ricoh5b_reg_info[] =
{
	{ "0x00", 0x00 },
	{ "0x01", 0x01 },
	{ "0x02", 0x02 },
	{ "0x03", 0x03 },
	{ "0x04", 0x04 },
	{ "0x05", 0x05 },
	{ "0x06", 0x06 },
	{ "0x07", 0x07 },
	{ "0x08", 0x08 },
	{ "0x09", 0x09 },
	{ "0x0A", 0x0A },
	{ "0x0B", 0x0B },
	{ "0x0C", 0x0C },
	{ "0x0D", 0x0D },
	{ "0x0E", 0x0E },
	{ "0x0F", 0x0F },
	{ "0x10", 0x10 },
	{ "0x11", 0x11 },
	{ "0x12", 0x12 },
	{ "0x13", 0x13 },
	{ "0x14", 0x14 },
	{ "0x15", 0x15 },
	{ "0x16", 0x16 },
	{ "0x17", 0x17 },
	{ "0x18", 0x18 },
	{ "0x19", 0x19 },
	{ "0x1A", 0x1A },
	{ "0x1B", 0x1B },
	{ "0x1C", 0x1C },
	{ "0x1D", 0x1D },
	{ "0x1E", 0x1E },
	{ "0x1F", 0x1F },
    { "0x20", 0x20 },
	{ "0x21", 0x21 },
	{ "0x22", 0x22 },
	{ "0x23", 0x23 },
	{ "0x24", 0x24 },
	{ "0x25", 0x25 },
	{ "0x26", 0x26 },
	{ "0x27", 0x27 },
	{ "0x28", 0x28 },
	{ "0x29", 0x29 },
	{ "0x2A", 0x2A },
	{ "0x2B", 0x2B },
	{ "0x2C", 0x2C },
	{ "0x2D", 0x2D },
	{ "0x2E", 0x2E },
	{ "0x2F", 0x2F },
	{ "0x30", 0x30 },
	{ "0x31", 0x31 },
	{ "0x32", 0x32 },
	{ "0x33", 0x33 },
	{ "0x34", 0x34 },
	{ "0x35", 0x35 },
	{ "0x36", 0x36 },
	{ "0x37", 0x37 },
	{ "0x38", 0x38 },
	{ "0x39", 0x39 },
	{ "0x3A", 0x3A },
	{ "0x3B", 0x3B },
	{ "0x3C", 0x3C },
	{ "0x3D", 0x3D },
	{ "0x3E", 0x3E },
	{ "0x3F", 0x3F },
	{ "0x40", 0x40 },
	{ "0x41", 0x41 },
	{ "0x42", 0x42 },
	{ "0x43", 0x43 },
	{ "0x44", 0x44 },
	{ "0x45", 0x45 },
	{ "0x46", 0x46 },
	{ "0x47", 0x47 },
	{ "0x48", 0x48 },
	{ "0x49", 0x49 },
	{ "0x4A", 0x4A },
	{ "0x4B", 0x4B },
	{ "0x4C", 0x4C },
	{ "0x4D", 0x4D },
	{ "0x4E", 0x4E },
	{ "0x4F", 0x4F },
	{ "0x50", 0x50 },
	{ "0x51", 0x51 },
	{ "0x52", 0x52 },
	{ "0x53", 0x53 },
	{ "0x54", 0x54 },
	{ "0x55", 0x55 },
	{ "0x56", 0x56 },
	{ "0x57", 0x57 },
	{ "0x58", 0x58 },
	{ "0x59", 0x59 },
	{ "0x5A", 0x5A },
	{ "0x5B", 0x5B },
	{ "0x5C", 0x5C },
	{ "0x5D", 0x5D },
	{ "0x5E", 0x5E },
	{ "0x5F", 0x5F },
	{ "0x60", 0x60 },
	{ "0x61", 0x61 },
	{ "0x62", 0x62 },
	{ "0x63", 0x63 },
	{ "0x64", 0x64 },
	{ "0x65", 0x65 },
	{ "0x66", 0x66 },
	{ "0x67", 0x67 },
	{ "0x68", 0x68 },
	{ "0x69", 0x69 },
	{ "0x6A", 0x6A },
	{ "0x6B", 0x6B },
	{ "0x6C", 0x6C },
	{ "0x6D", 0x6D },
	{ "0x6E", 0x6E },
	{ "0x6F", 0x6F },
	{ "0x70", 0x70 },
	{ "0x71", 0x71 },
	{ "0x72", 0x72 },
	{ "0x73", 0x73 },
	{ "0x74", 0x74 },
	{ "0x75", 0x75 },
	{ "0x76", 0x76 },
	{ "0x77", 0x77 },
	{ "0x78", 0x78 },
	{ "0x79", 0x79 },
	{ "0x7A", 0x7A },
	{ "0x7B", 0x7B },
	{ "0x7C", 0x7C },
	{ "0x7D", 0x7D },
	{ "0x7E", 0x7E },
	{ "0x7F", 0x7F },
	{ "0x80", 0x80 },
	{ "0x81", 0x81 },
	{ "0x82", 0x82 },
	{ "0x83", 0x83 },
	{ "0x84", 0x84 },
	{ "0x85", 0x85 },
	{ "0x86", 0x86 },
	{ "0x87", 0x87 },
	{ "0x88", 0x88 },
	{ "0x89", 0x89 },
	{ "0x8A", 0x8A },
	{ "0x8B", 0x8B },
	{ "0x8C", 0x8C },
	{ "0x8D", 0x8D },
	{ "0x8E", 0x8E },
	{ "0x8F", 0x8F },
	{ "0x90", 0x90 },
	{ "0x91", 0x91 },
	{ "0x92", 0x92 },
	{ "0x93", 0x93 },
	{ "0x94", 0x94 },
	{ "0x95", 0x95 },
	{ "0x96", 0x96 },
	{ "0x97", 0x97 },
	{ "0x98", 0x98 },
	{ "0x99", 0x99 },
	{ "0x9A", 0x9A },
	{ "0x9B", 0x9B },
	{ "0x9C", 0x9C },
	{ "0x9D", 0x9D },
	{ "0x9E", 0x9E },
	{ "0x9F", 0x9F },
	{ "0xA0", 0xA0 },
	{ "0xA1", 0xA1 },
	{ "0xA2", 0xA2 },
	{ "0xA3", 0xA3 },
	{ "0xA4", 0xA4 },
	{ "0xA5", 0xA5 },
	{ "0xA6", 0xA6 },
	{ "0xA7", 0xA7 },
	{ "0xA8", 0xA8 },
	{ "0xA9", 0xA9 },
	{ "0xAA", 0xAA },
	{ "0xAB", 0xAB },
	{ "0xAC", 0xAC },
	{ "0xAD", 0xAD },
	{ "0xAE", 0xAE },
	{ "0xAF", 0xAF },
	{ "0xB0", 0xB0 },
	{ "0xB1", 0xB1 },
	{ "0xB2", 0xB2 },
	{ "0xB3", 0xB3 },
	{ "0xB4", 0xB4 },
	{ "0xB5", 0xB5 },
	{ "0xB6", 0xB6 },
	{ "0xB7", 0xB7 },
	{ "0xB8", 0xB8 },
	{ "0xB9", 0xB9 },
	{ "0xBA", 0xBA },
	{ "0xBB", 0xBB },
	{ "0xBC", 0xBC },
	{ "0xBD", 0xBD },
	{ "0xBE", 0xBE },
	{ "0xBF", 0xBF },
	{ "0xC0", 0xC0 },
	{ "0xC1", 0xC1 },
	{ "0xC2", 0xC2 },
	{ "0xC3", 0xC3 },
	{ "0xC4", 0xC4 },
	{ "0xC5", 0xC5 },
	{ "0xC6", 0xC6 },
	{ "0xC7", 0xC7 },
	{ "0xC8", 0xC8 },
	{ "0xC9", 0xC9 },
	{ "0xCA", 0xCA },
	{ "0xCB", 0xCB },
	{ "0xCC", 0xCC },
	{ "0xCD", 0xCD },
	{ "0xCE", 0xCE },
	{ "0xCF", 0xCF },
	{ "0xD0", 0xD0 },
	{ "0xD1", 0xD1 },
	{ "0xD2", 0xD2 },
	{ "0xD3", 0xD3 },
	{ "0xD4", 0xD4 },
	{ "0xD5", 0xD5 },
	{ "0xD6", 0xD6 },
	{ "0xD7", 0xD7 },
	{ "0xD8", 0xD8 },
	{ "0xD9", 0xD9 },
	{ "0xDA", 0xDA },
	{ "0xDB", 0xDB },
	{ "0xDC", 0xDC },
	{ "0xDD", 0xDD },
	{ "0xDE", 0xDE },
	{ "0xDF", 0xDF },
	{ "0xE0", 0xE0 },
	{ "0xE1", 0xE1 },
	{ "0xE2", 0xE2 },
	{ "0xE3", 0xE3 },
	{ "0xE4", 0xE4 },
	{ "0xE5", 0xE5 },
	{ "0xE6", 0xE6 },
	{ "0xE7", 0xE7 },
	{ "0xE8", 0xE8 },
	{ "0xE9", 0xE9 },
	{ "0xEA", 0xEA },
	{ "0xEB", 0xEB },
	{ "0xEC", 0xEC },
	{ "0xED", 0xED },
	{ "0xEE", 0xEE },
	{ "0xEF", 0xEF },
	{ "0xF0", 0xF0 },
	{ "0xF1", 0xF1 },
	{ "0xF2", 0xF2 },
	{ "0xF3", 0xF3 },
	{ "0xF4", 0xF4 },
	{ "0xF5", 0xF5 },
	{ "0xF6", 0xF6 },
	{ "0xF7", 0xF7 },
	{ "0xF8", 0xF8 },
	{ "0xF9", 0xF9 },
	{ "0xFA", 0xFA },
	{ "0xFB", 0xFB },
	{ "0xFC", 0xFC },
	{ "0xFD", 0xFD },
	{ "0xFE", 0xFE },
	{ "0xFF", 0xFF },
};

static regmon_customer_info_t ricoh5b_customer_info =
{
	.name           = "ricoh5b_tuner",
	.reg_info       = ricoh5b_reg_info,
	.reg_info_count = sizeof(ricoh5b_reg_info)/sizeof(regmon_reg_info_t),
	.write_reg      = ricoh5b_regmon_write_reg,
	.read_reg       = ricoh5b_regmon_read_reg,
	.private_data   = NULL,
};

static int ricoh5b_regmon_add(void *data)
{
	ricoh5b_customer_info.private_data=data;
	regmon_add(&ricoh5b_customer_info);
	return(0);
}

static int ricoh5b_regmon_del(void)
{
	regmon_del(&ricoh5b_customer_info);
	return(0);
}

static int ricoh5b_regmon_write_reg(void *private_data, unsigned int address, unsigned int value)
{
    int ret = 0;
    unsigned char buf_w[WRITE_BUF_SIZE] = {0, };
    unsigned char add_w = 0;
    
    add_w = (unsigned char)value;
    memcpy(buf_w, &add_w, 1);

    ret = ricoh5b_cmd_write_byte((unsigned char)address, buf_w, 2);

    return ret;

}

static int ricoh5b_regmon_read_reg(void *private_data, unsigned int address, unsigned int *value)
{
    int ret = 0;
    unsigned char buf_r[READ_BUF_SIZE] = {0, };
    
    ret = ricoh5b_cmd_read_byte((unsigned char)address, buf_r);
    
    *value=buf_r[0];
    
	return ret;
}

#endif

/******************************************************************************/
/*! @brief get AGCDET offset
*//****************************************************************************/
unsigned char ricoh5b_common_cmd_get_det_offset(void)
{
	return det_offset;
}

/******************************************************************************/
/*! @brief wait timer API
*//****************************************************************************/
void ricoh5b_cmd_set_sequence_timer(unsigned long timeout_us)
{
	usleep_range(timeout_us, timeout_us*2);
}

/******************************************************************************/
/*! @brief immunity measure Enable/Disable status get
*//****************************************************************************/
RICOH5B_IMMUNITY ricoh5b_cmd_get_immunity_measure(void)
{
	#if 0	/* At 2014/09/12, not use below */
	if (p_get_immunity_measure != (RICOH5B_IMMUNITY (*)(void))RICOH5B_NULL) {
		return p_get_immunity_measure();
	} else {
		return RICOH5B_IMMUNITY_UNKNOWN;
	}
	#else
		return RICOH5B_IMMUNITY_UNKNOWN;
	#endif
}

/******************************************************************************/
/*! @brief Image Rejection Phase/Gain correction address
*//****************************************************************************/
static unsigned char get_pg_correction_address(unsigned char pgm)
{
	if (pgm <= RICOH5B_REG_DAT_PGM_4DB) {
		return RICOH5B_REG_ADR_RGCOR_0_L;
	} else if (pgm <= RICOH5B_REG_DAT_PGM_8DB) {
		return RICOH5B_REG_ADR_RGCOR_1_L;
	} else if (pgm <= RICOH5B_REG_DAT_PGM_14DB) {
		return RICOH5B_REG_ADR_RGCOR_2_L;
	} else if (pgm <= RICOH5B_REG_DAT_PGM_20DB) {
		return RICOH5B_REG_ADR_RGCOR_3_L;
	} else {
		return RICOH5B_REG_ADR_RGCOR_4_L;
	}
}

/******************************************************************************/
/*! @brief set Phase/Gain table command sequence
*//****************************************************************************/
static int set_pg_tbl_cmd_sequence(void)
{
	unsigned char data[RICOH5B_REG_LENGTH_PG];
	unsigned char address;
	unsigned short gain;
	unsigned short phase;
	int ret = 0;
	
	if (p_fine_pg_step < RICOH5B_AGCG_NUM) {
		/* set Phase/Gain correction */
		/* PRQA S 506 */
		if ((0x400 <= p_fine_pg->gain[p_fine_pg_step]) && (p_fine_pg->gain[p_fine_pg_step] <= 0xC00)) {	
			gain	= p_fine_pg->gain[p_fine_pg_step];
			phase = p_fine_pg->phase[p_fine_pg_step];
		} else {
			gain	= RICOH5B_REG_DAT_RGCOR_DEFAULT;
			phase = RICOH5B_REG_DAT_RPHCOR_DEFAULT;
		}
		
		address = get_pg_correction_address(align_agcg_tbl[p_fine_pg_step]);
		data[0] = RICOH5B_GET_LOWER_BYTE(gain);
		data[1] = RICOH5B_GET_UPPER_BYTE(gain);
		data[2] = RICOH5B_GET_LOWER_BYTE(phase);
		data[3] = RICOH5B_GET_UPPER_BYTE(phase);
		
		ret = ricoh5b_cmd_write_byte(address, data, RICOH5B_REG_LENGTH_PG);
		p_fine_pg_step++;
	}
	
	return ret;
}

/******************************************************************************/
/*! @brief set Phase/Gain table command
*//****************************************************************************/
int ricoh5b_send_set_pg_tbl(RICOH5B_PG *p_pg)
{
    int ret = 0;

	if (p_pg != (RICOH5B_PG *)RICOH5B_NULL) {
		p_fine_pg = p_pg;
		ret = set_pg_tbl_cmd_sequence();
	}
	return ret;
}

/******************************************************************************/
/*! @brief set De-Emphasis command
*//****************************************************************************/
static int ricoh5b_send_set_de_emphasis(struct ricoh5b_dev *ricoh5b_dev)
{
    int ret = 0;

	ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_DEMP,    ricoh5b_dev->deemp,    RICOH5B_REG_BIT_DEMP);
	if(ret < 0){
	    pr_err("%s:RICOH5B_REG_BIT_DEMP err = %d \n",__func__, ret);
	    goto send_err;
	}

	ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_DEMP_CE, ricoh5b_dev->deemp_ce, RICOH5B_REG_BIT_DEMP_CE);
	if(ret < 0){
	    pr_err("%s:RICOH5B_REG_BIT_DEMP_CE err = %d \n",__func__, ret);
	    goto send_err;
	}

	return 0;

send_err:
    return -1;
}

/******************************************************************************/
/*! @brief Write Packet Create Function
*//****************************************************************************/
static void generate_write_packet(unsigned char *buf, unsigned char reg_addr, unsigned char *data, unsigned char count)
{
	int i;
	
	if ((count <= DATA_WRITE_LENGTH_MAX) && (data != (unsigned char *)RICOH5B_NULL)) {
		for (i = count-1; i >= 0; i--) {
			buf[WRITE_INDEX_DATA+i] = data[i];
		}
		buf[WRITE_INDEX_ADDRESS] = reg_addr;
	}
}

/******************************************************************************/
/*! @brief Data Read Function
*//****************************************************************************/
int ricoh5b_cmd_read_byte(unsigned char reg_addr, unsigned char *buf)
{
	struct i2c_client *ricoh5b_client;
	int ret = 0;
	int retry = RICOH5B_I2C_RETRY_COUNT;

    ricoh5b_client = ricoh5b_i2c_client;
    
	do {
		retry--;

        ricoh5b_client->ext_flag = 0;
        ricoh5b_client->ext_flag = ((ricoh5b_client->ext_flag & I2C_MASK_FLAG) | I2C_RS_FLAG |I2C_WR_FLAG |
        I2C_DIRECTION_FLAG);
        ricoh5b_client->timing = 0;

        buf[0] = reg_addr;

        ret = i2c_master_send(ricoh5b_client, buf, MTK_I2C_READ_WRITE_LEN(1,1));

        if(ret < 0){
        	pr_err("%s:I2C Read error ret = %d addr=0x%x val = 0x%x retry = %d \n",__func__, ret, ricoh5b_client->addr, buf[0],  retry);
        }else{
            break;
        }

		ricoh5b_cmd_set_sequence_timer(6000); /* Retry, chip was in standby */
    } while (retry);

	#ifdef RICOH5B_MAIN_DEBUG
    pr_info("%s: addr=0x%x buf = 0x%x \n",__func__, reg_addr,buf[0]);
	#endif
	
    ricoh5b_client->ext_flag = 0;

	return ret;
}

/******************************************************************************/
/*! @brief Multi Byte data Write API
 *
 *  @param  reg_addr  [in]  Register Address
 *  @param  data      [in]  First Address of Data buffer
 *  @param  count     [in]  (Address + Data) Length
 *
*//****************************************************************************/
int ricoh5b_cmd_write_byte(unsigned char reg_addr, unsigned char *buf, size_t count)
{
	int ret = 0, retry = RICOH5B_I2C_RETRY_COUNT;
	struct i2c_client *ricoh5b_client;
	#ifdef RICOH5B_MAIN_DEBUG
	unsigned char buf_read[READ_BUF_SIZE] = {0, };
	#endif

	generate_write_packet(buf, reg_addr, buf, count - 1);
	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("ricoh5b_cmd_write_byte: buf[0]=%x, buf[1]=%x\n", buf[0], buf[1]);
	#endif

	ricoh5b_client = ricoh5b_i2c_client;

	/* Write data */
	do {
		retry--;
		
		ricoh5b_client->ext_flag = 0;
		ricoh5b_client->timing = 0;
		
		ret = i2c_master_send(ricoh5b_i2c_client, buf, count);

		if (ret == count){
			#ifdef RICOH5B_MAIN_DEBUG
		    ricoh5b_cmd_read_byte(reg_addr, buf_read);
		    #endif
		    break;
		}
		ricoh5b_cmd_set_sequence_timer(6000); /* Retry, chip was in standby */
	} while (retry);
	
	return ret;
}

/******************************************************************************/
/*! @brief Mask 1Byte data Write API
 *
 *  @param  reg_addr  [in]  Register Address
 *  @param  mask      [in]  Bit Mask
 *  @param  data      [in]  Data
 *
*//****************************************************************************/
int ricoh5b_cmd_write_bit(unsigned char reg_addr, unsigned char data, unsigned char mask)
{
	int ret = 0;
	unsigned char buf[WRITE_BUF_SIZE] = {0, };

	ret = ricoh5b_cmd_read_byte(reg_addr, buf); /* Read current register value */
	if (ret < 0) {
		pr_err("%s : ricoh5b_cmd_read_byte returned %d\n", __func__, ret);
		ret = -EIO;
		return ret;
	}
	
	buf[0] &= (unsigned char)(~mask);	/* clear target bit	*/
	buf[0] |= (data & mask);			/* add target bit */
	
	ret = ricoh5b_cmd_write_byte(reg_addr, buf, RICOH5B_REG_ADDRESS_LENGTH + DATA_1BYTE_LENGTH); /* Write data */
	if (ret != RICOH5B_REG_ADDRESS_LENGTH + DATA_1BYTE_LENGTH) {
		pr_err("%s : i2c_master_send returned %d\n", __func__, ret);
		ret = -EIO;
	}
	
	return ret;
}

/******************************************************************************/
/*! @brief table set data sequencial Write API
 *
 *  @param  tbl    [in]  Write Data Table
 *  @param  size   [in]  Table Size
 *
*//****************************************************************************/
int ricoh5b_cmd_write_sequencial(RICOH5B_REG *tbl, unsigned char count)
{
	int ret = 0;
	unsigned char w_tbl_index = 0;/* Sequencial Write table index */
	unsigned char buf[WRITE_BUF_SIZE] = {0, };

	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("ricoh5b_cmd_write_sequencial: count=%d\n", count);
	#endif
	for(w_tbl_index = 0; w_tbl_index < count; w_tbl_index++){
		/* write */
		if (tbl[w_tbl_index].mask == RICOH5B_REG_NON_MASK) {	/* write byte */
			memcpy(buf, &(tbl[w_tbl_index].data), DATA_1BYTE_LENGTH);
			ret = ricoh5b_cmd_write_byte(tbl[w_tbl_index].address, buf, RICOH5B_REG_ADDRESS_LENGTH + DATA_1BYTE_LENGTH);
			if (ret != RICOH5B_REG_ADDRESS_LENGTH + DATA_1BYTE_LENGTH) {
				pr_err("%s : ricoh5b_cmd_write_byte returned %d\n", __func__, ret);
				ret = -EIO;
				return ret;
			}
		} else {	/* write bit */
			ret = ricoh5b_cmd_write_bit(tbl[w_tbl_index].address, tbl[w_tbl_index].data, tbl[w_tbl_index].mask);
			if (ret != RICOH5B_REG_ADDRESS_LENGTH + DATA_1BYTE_LENGTH) {
				pr_err("%s : ricoh5b_cmd_write_bit returned %d\n", __func__, ret);
				ret = -EIO;
				return ret;
			}
		}
	}

	return ret;
}

/******************************************************************************/
/*! @brief audio output setting command
 *
 *  @param  kind  [in]  Analog      -> 0 :RICOH5B_AUDIO_OUTPUT_ANALOG<br>
              IIS 32fs mode  -> 1 :RICOH5B_AUDIO_OUTPUT_IIS_32FS<br>
              IIS 64fs mode  -> 2 :RICOH5B_AUDIO_OUTPUT_IIS_64FS<br>
*//****************************************************************************/
static int ricoh5b_send_audio_output_setting(unsigned char kind)
{
    int ret = 0;

	switch (kind) {
	case RICOH5B_AUDIO_OUTPUT_ANALOG:
		ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_audio_output_analog_tbl[0], RICOH5B_CMD_AUDIO_OUTPUT_ANALOG_TBL_SIZE);
		break;
	case RICOH5B_AUDIO_OUTPUT_IIS_32FS:
		ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_audio_output_iis_32fs_tbl[0], RICOH5B_CMD_AUDIO_OUTPUT_IIS_32FS_TBL_SIZE);
		break;
	case RICOH5B_AUDIO_OUTPUT_IIS_64FS:
		ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_audio_output_iis_64fs_tbl[0], RICOH5B_CMD_AUDIO_OUTPUT_IIS_64FS_TBL_SIZE);
		break;
	default:
		break;
	}
	
	return ret;
}

/******************************************************************************/
/*! @brief Tuner Initialize sequence
 *
*//****************************************************************************/
static int ricoh5b_initialize_setting(void)
{
	int step;
	int ret = 0;

	for(step = 0; step < P_ON_STEP_END; step++){
		#ifdef RICOH5B_MAIN_DEBUG
		pr_info("IOCTL ricoh5b Initialize Step == %d\n", step);
		#endif
		switch (step) {
		case P_ON_STEP_REG_RESET:
			ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_reg_reset_tbl[0], RICOH5B_CMD_REG_RESET_TBL_SIZE);
			if(ret < 0){
	            pr_err("%s:RICOH5B_CMD_REG_RESET_TBL err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_MODULE_ON:
			ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_module_power_on_tbl[0], RICOH5B_CMD_MODULE_POWER_ON_TBL_SIZE);
			if(ret < 0){
	            pr_err("%s:RICOH5B_CMD_MODULE_POWER_ON_TBL err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_XTAL_SETTING:
			ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_xtal_setting_tbl[0], RICOH5B_CMD_XTAL_SETTING_TBL_SIZE);
			if(ret < 0){
	            pr_err("%s:RICOH5B_CMD_XTAL_SETTING_TBL err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_SYNTHE_SETTING:
			ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_synthe_setting_tbl[0], RICOH5B_CMD_SYNTHE_SETTING_TBL_SIZE);
			if(ret < 0){
	            pr_err("%s:RICOH5B_CMD_SYNTHE_SETTING_TBL err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_AGC_SETTING:
			ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_agc_setting_tbl[0], RICOH5B_CMD_AGC_SETTING_TBL_SIZE);
			if(ret < 0){
	            pr_err("%s:RICOH5B_CMD_AGC_SETTING_TBL err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_DSP_INIT_SETTING:
			ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_dsp_init_setting_tbl[0], RICOH5B_CMD_DSP_INIT_SETTING_TBL_SIZE);
			if(ret < 0){
	            pr_err("%s:RICOH5B_CMD_DSP_INIT_SETTING_TBL err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_CHIP_ON:
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_PE, RICOH5B_REG_DAT_PE_ON, RICOH5B_REG_BIT_PE);
			if(ret < 0){
	            pr_err("%s:P_ON_STEP_CHIP_ON err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_WAIT_CHIP_ON:
			ricoh5b_cmd_set_sequence_timer(CHIP_POWER_ON_WAIT_TIME);
			break;
		case P_ON_STEP_BLOCK_ON:
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_BLOCK1_PE, RICOH5B_REG_DAT_BLOCK1_PE_ALL_ON, RICOH5B_REG_NON_MASK);
			if(ret < 0){
	            pr_err("%s:P_ON_STEP_BLOCK_ON err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_WAIT_BLOCK_ON:
            ricoh5b_cmd_set_sequence_timer(BLOCK_POWER_ON_WAIT_TIME);
			break;
		case P_ON_STEP_CK_GEN_SETTING:
			ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_clk_setting_tbl[0], RICOH5B_CMD_CLK_SETTING_TBL_SIZE);
			if(ret < 0){
	            pr_err("%s:RICOH5B_CMD_CLK_SETTING_TBL err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_RDSI_SETTING:
			ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_rdsi_output_tbl[0], RICOH5B_CMD_RDSI_OUTPUT_TBL_SIZE);
			if(ret < 0){
	            pr_err("%s:RICOH5B_CMD_RDSI_OUTPUT_TBL err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_AUDIO_SETTING:
			ret = ricoh5b_send_audio_output_setting(RICOH5B_SELECT_AUDIO_OUTPUT);
			if(ret < 0){
	            pr_err("%s:P_ON_STEP_AUDIO_SETTING err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_RTRIM_SETTING:
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_RTRIM_SEL, RICOH5B_REG_DAT_RTRIM_SEL_MANUAL, RICOH5B_REG_NON_MASK);
			if(ret < 0){
	            pr_err("%s:P_ON_STEP_RTRIM_SETTING err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case P_ON_STEP_END:
		default:
			break;
		}
	}
	
	return 0;

send_err:
    return ret;
}

/******************************************************************************/
/*! @brief Tuner audio output mute on sequence
 *
*//****************************************************************************/
static int ricoh5b_audio_output_mute_on(void)
{
	int step;
	int ret = 0;

	for(step=0; step<MUTE_ON_STEP_END; step++){
		#ifdef RICOH5B_MAIN_DEBUG
		pr_info("IOCTL ricoh5b Mute ON Step == %d\n", step);
		#endif
		switch (step) {
		case MUTE_ON_STEP_SET_WAIT_COUNT:
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_MUTE_WAIT_CNT, RICOH5B_REG_DAT_MUTE_WAIT_CNT_MUTE_ON, RICOH5B_REG_NON_MASK);
			if(ret < 0){
	            pr_err("%s:MUTE_ON_STEP_SET_WAIT_COUNT err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case MUTE_ON_STEP_WAIT_WAIT_COUNT:
			ricoh5b_cmd_set_sequence_timer(MUTING_WAIT_TIME);
			break;
		case MUTE_ON_STEP_MUTE_ON:
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_MUTE, RICOH5B_REG_DAT_MUTE_ON, RICOH5B_REG_BIT_MUTE);
			if(ret < 0){
	            pr_err("%s:MUTE_ON_STEP_MUTE_ON err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case MUTE_ON_STEP_WAIT_MUTE_ON:
			ricoh5b_cmd_set_sequence_timer(MUTING_WAIT_TIME);
			if(ret < 0){
	            pr_err("%s:MUTE_ON_STEP_WAIT_MUTE_ON err = %d \n",__func__, ret);
	            goto send_err;
	        }
			break;
		case MUTE_ON_STEP_END:
		default:
			break;
		}
	}

	return 0;
send_err:
    return -1;
}

/******************************************************************************/
/*! @brief Tuner audio output mute off sequence
 *
*//****************************************************************************/
int ricoh5b_audio_output_mute_off(void)
{
	int step;
	int ret = 0;

	for(step = 0; step < MUTE_OFF_STEP_END; step++){
		#ifdef RICOH5B_MAIN_DEBUG
		pr_info("IOCTL ricoh5b Mute OFF Step == %d\n", step);
		#endif
		switch (step) {
		case MUTE_OFF_STEP_MUTE_OFF:
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_MUTE, RICOH5B_REG_DAT_MUTE_OFF, RICOH5B_REG_BIT_MUTE);
			if(ret < 0){
			    pr_err("%s:MUTE_OFF_STEP_MUTE_OFF err = %d \n",__func__, ret);
	            goto send_err;
			}
			break;
		case MUTE_OFF_STEP_WAIT_MUTE_OFF:
			ricoh5b_cmd_set_sequence_timer(MUTING_WAIT_TIME);
			break;
		case MUTE_OFF_STEP_SET_WAIT_COUNT:
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_MUTE_WAIT_CNT, RICOH5B_REG_DAT_MUTE_WAIT_CNT_MUTE_OFF, RICOH5B_REG_NON_MASK);
			if(ret < 0){
			    pr_err("%s:MUTE_OFF_STEP_SET_WAIT_COUNT err = %d \n",__func__, ret);
	            goto send_err;
			}
			break;
		case MUTE_OFF_STEP_END:
		default:
			break;
		}
	}
	return 0;

send_err:
    return -1;
}

/******************************************************************************/
/*! @brief DITHER (modulater DC-offset) setting command
 *
 *  @param  band    [in]  FM/AM/IDLE
 *          audio_type  [in]  Analog      -> 0 :RICOH5B_AUDIO_OUTPUT_ANALOG<br>
                IIS 32fs mode  -> 1 :RICOH5B_AUDIO_OUTPUT_IIS_32FS<br>
                IIS 64fs mode  -> 2 :RICOH5B_AUDIO_OUTPUT_IIS_64FS<br>
*//****************************************************************************/
int ricoh5b_send_dither_setting(RICOH5B_BAND_STATUS band, unsigned char audio_type)
{
    int ret = 0;
	if (audio_type == RICOH5B_AUDIO_OUTPUT_ANALOG) {
		/* Analog audio */
		if (band == RICOH5B_BAND_FM) {
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_DITHER, RICOH5B_REG_DAT_DITHER_AUDIO_ANALOG_FM, RICOH5B_REG_NON_MASK);
		} else if (band == RICOH5B_BAND_AM) {
			ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_DITHER, RICOH5B_REG_DAT_DITHER_AUDIO_ANALOG_AM, RICOH5B_REG_NON_MASK);
		} else {
			/* do nothing */
		}
	} else if ((audio_type == RICOH5B_AUDIO_OUTPUT_IIS_32FS) || (audio_type == RICOH5B_AUDIO_OUTPUT_IIS_64FS)) {
		/* Digital audio */
		ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_DITHER, RICOH5B_REG_DAT_DITHER_AUDIO_IIS, RICOH5B_REG_NON_MASK);
	} else {
		/* do nothing */
	}
	return ret;
}

static int ricoh5b_power_stop(void)
{
    int ret = 0;

	ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_MUTE_WAIT_CNT, RICOH5B_REG_DAT_MUTE_WAIT_CNT_MUTE_ON, RICOH5B_REG_NON_MASK);
	if(ret < 0){
        pr_err("%s:RICOH5B_REG_ADR_MUTE_WAIT_CNT err = %d \n",__func__, ret);
        goto send_err;
    }
	msleep(20);
	ret = ricoh5b_cmd_write_bit(RICOH5B_REG_ADR_MUTE, RICOH5B_REG_DAT_MUTE_ON, RICOH5B_REG_BIT_MUTE);
	if(ret < 0){
        pr_err("%s:RICOH5B_REG_ADR_MUTE err = %d \n",__func__, ret);
        goto send_err;
    }
	msleep(20);
	
	ret = ricoh5b_cmd_write_sequencial((RICOH5B_REG *)&ricoh5b_cmd_chip_power_off_tbl[0], RICOH5B_CMD_CHIP_POWER_OFF_TBL_SIZE);
	if(ret < 0){
        pr_err("%s:RICOH5B_CMD_CHIP_POWER_OFF_TBL err = %d \n",__func__, ret);
        goto send_err;
    }

	return 0;

send_err:
    return ret;
}

/***********************/
/*@ operation_routines */
/***********************/
static void ricoh5b_repeat_check(struct work_struct * work)
{
    struct ricoh5b_dev *ricoh5b_dev;
    int audmode = 0;
    int signal_strength = 0;
    int ret = 0;

    if(tuner_open == 0){
        return;
    }
   
    ricoh5b_dev = ricoh5b_device_info;
    
    mutex_lock(&ricoh5b_dev->repeat_check);
    
    ret = ricoh5b_rec_audio_mode();
    if(ret < 0){
        pr_err("%s: Stereo read Err\n",__func__);
    }else{
        audmode = ret;
    }

    ret = ricoh5b_station_check(ricoh5b_dev);
    
    if(ret < 0){
        signal_strength = 0;
    }else{
        signal_strength = 1;
    }
    
    if(audmode != ricoh5b_dev->rec_audio_mode){
        change_audmode = POLL_CHECK_FLAG_ENABLE;
        ricoh5b_dev->rec_audio_mode = audmode;
    }
    
    if(signal_strength != res_signal_strength){
        change_signal_strength = POLL_CHECK_FLAG_ENABLE;
        res_signal_strength = signal_strength;
    }
    
    if(change_audmode == POLL_CHECK_FLAG_ENABLE || change_signal_strength == POLL_CHECK_FLAG_ENABLE){
        wake_up_interruptible(&ricoh5b_dev->read_wq);
    }

    mutex_unlock(&ricoh5b_dev->repeat_check);

    if(tuner_open == 1){
        queue_delayed_work(
            workqueue_check,
            &ricoh5b_check_work,
            RICOH5B_STATE_POLL_TIME
        );
    }

    return;
}

static int ricoh5b_rec_audio_mode(void)
{
    int ret = 0;
    unsigned char buf_stereo[READ_BUF_SIZE] = {0, };

    ret = ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_STEREO, buf_stereo);
	if(ret < 0){
	    pr_err("%s:ricoh5b_stereo_check Error\n",__func__);
	    return -1;
	}
	
    ret = buf_stereo[0] & RICOH5B_REG_BIT_STEREO;
	
	return ret;
}

static int ricoh5b_fo_sat_check(void)
{
    unsigned char buf_status[READ_BUF_SIZE] = {0, };
    int ret = 0;
   
    ret = ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_FO_SAT, buf_status);
    if(ret < 0){
        pr_err("%s:read err = %d \n",__func__, ret);
        goto read_err;
    }
    
    if((buf_status[0] & RICOH5B_REG_BIT_FO_SAT) == RICOH5B_REG_DAT_FO_SAT_INSIDE){
        return 0;
    }

read_err:
    return -1;
}

static int ricoh5b_rssi_check(int sens, int *value)
{
    unsigned char buf_status[READ_BUF_SIZE] = {0, };
    int ret = 0;
    int rssi_rd = -1;

    ret = ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RSSI_RD, buf_status);
    if(ret < 0){
        pr_err("%s:read err = %d \n",__func__, ret);
        goto read_err;
    }
    rssi_rd = buf_status[0] & RICOH5B_SIGNAL_STRENGTH_MASK;

    if(rssi_rd < sens){
        return -1;
    }

    *value = rssi_rd;

    return 0;

read_err:
    return -1;    
}

static int ricoh5b_station_check(struct ricoh5b_dev *ricoh5b_dev)
{
    int ret = 0;
    int retry = 0;
    int rssi_raw = 0;
    
    msleep(80);
    
    for(retry = 0; retry < SIGNAL_STRENGTH_RETRY; retry++){
        
        msleep(20);

        ret = ricoh5b_fo_sat_check();
        if(ret){
            #ifdef RICOH5B_MAIN_DEBUG
            pr_err("%s: fo_sat Error\n",__func__);
            #endif
            goto check_ng;
        }

        ret = ricoh5b_rssi_check(RICOH5B_RSSI_THRESHOLD, &rssi_raw);
        if(ret){
            #ifdef RICOH5B_MAIN_DEBUG
            pr_err("%s:rssi err \n", __func__);
            #endif
            goto check_ng;
        }
    }

    mutex_lock(&ricoh5b_dev->station_mutex);
    ricoh5b_dev->station_check = 1;
    mutex_unlock(&ricoh5b_dev->station_mutex);

    #ifdef RICOH5B_MAIN_DEBUG
    pr_info("%s: station check OK rssi_raw=%d \n", __func__, rssi_raw);
    #endif

    return 0;
    
check_ng:
    mutex_lock(&ricoh5b_dev->station_mutex);
    ricoh5b_dev->station_check = 0;
    mutex_unlock(&ricoh5b_dev->station_mutex);
    return -1;
}

static unsigned char buf_rds_data[RICOH5B_RDS_BUF];
static unsigned char buf_rds_check[RICOH5B_RDS_BUF];
static unsigned char buf_rds_resdata[RICOH5B_RDS_BUF];

static void ricoh5b_next_rds_eint(struct ricoh5b_dev *ricoh5b_dev)
{
    mutex_lock(&ricoh5b_dev->rds_mode_mutex);
    if(ricoh5b_dev->rds_mode == RICOH5B_RDS_MODE_ON){
        mt_eint_unmask(RICOH5B_RDSI_EINT);
    }else{
        mt_eint_mask(RICOH5B_RDSI_EINT);
    }
    mutex_unlock(&ricoh5b_dev->rds_mode_mutex);
    return;
}

static void ricoh5b_get_rds(struct work_struct * work)
{
    struct ricoh5b_dev *ricoh5b_dev;
    unsigned char buf_status[READ_BUF_SIZE] = {0, };
    unsigned char buf_data_l[READ_BUF_SIZE] = {0, };
    unsigned char buf_data_h[READ_BUF_SIZE] = {0, };
    unsigned char buf_offset_l[READ_BUF_SIZE] = {0, };
    unsigned char buf_offset_h[READ_BUF_SIZE] = {0, };
    unsigned char rds_noerr = 0;
    unsigned char rds_unc = 0;
    unsigned char rds_block = 0;
    unsigned char blocktype = 0;
    unsigned char segment_addr = 0;
    static int write_pos = 0;
    int i = 0;
    int gpio_val = 0;
    static int next_data_write = RDS_WRITE_DISABLE;

    ricoh5b_dev = ricoh5b_device_info;

	gpio_val = mt_get_gpio_in(ricoh5b_dev->iic_gpio);
	
	if(gpio_val == 0){
        ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_STATUS, buf_status);
        ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_DATA_L, buf_data_l);
        ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_DATA_H, buf_data_h);
        ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_OFFSET_L, buf_offset_l);
        ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_OFFSET_H, buf_offset_h);
        mutex_lock(&ricoh5b_dev->station_mutex);
        if(ricoh5b_dev->station_check == 0){
            pr_err("%s: ricoh5b_dev->station_check Error !!!\n", __func__);
            next_data_write = RDS_WRITE_DISABLE;
            memset(&buf_rds_data, 0, sizeof(buf_rds_data));
            memset(&buf_rds_check, 0, sizeof(buf_rds_check));
            ricoh5b_next_rds_eint(ricoh5b_dev);
            mutex_unlock(&ricoh5b_dev->station_mutex);
            return;
        }
        mutex_unlock(&ricoh5b_dev->station_mutex);
        
        rds_noerr = buf_status[0] & RICOH5B_RDS_NOERR;
        rds_unc   = buf_status[0] & RICOH5B_RDS_UNC;
        rds_block = (buf_status[0] & RICOH5B_RDS_BLOCK) >> RDS_BLOCK_SHIFT;
        blocktype = buf_data_h[0] & RICOH5B_RDS_BLOCK_MASK;
        segment_addr = (buf_data_l[0] & RICOH5B_RDS_SEGMENT_MASK);

        #ifdef RICOH5B_MAIN_DEBUG
        pr_info("%s: rds_noerr = 0x%x \n",__func__, rds_noerr);
        pr_info("%s: rds_unc = 0x%x \n",__func__, rds_unc);
        pr_info("%s: rds_block = 0x%x \n",__func__, rds_block);
        pr_info("%s: blocktype = 0x%x \n",__func__, blocktype);
        pr_info("%s: buf_data_l[0] = 0x%x \n",__func__, buf_data_l[0]);
        pr_info("%s: buf_data_h[0] = 0x%x \n",__func__, buf_data_h[0]);
        pr_info("%s: segment addr = 0x%x \n",__func__, segment_addr);
        #endif

        if(rds_noerr == RICOH5B_RDS_NOERR || rds_unc == RICOH5B_RDS_UNC){
            if(rds_block == RICOH5B_RDS_BLOCK_B){
                if(blocktype == RICOH5B_RDS_BLOCK_B_PS){
                    if(segment_addr == RICOH5B_RDS_SEGMENT_ADDR_1 && write_pos == FIRST_POSITION){
                        next_data_write = RDS_WRITE_ENABLE;
                    }else if(segment_addr == RICOH5B_RDS_SEGMENT_ADDR_2 && write_pos == SECOND_POSITION){
                        next_data_write = RDS_WRITE_ENABLE;
                    }else if(segment_addr == RICOH5B_RDS_SEGMENT_ADDR_3 && write_pos == THIRD_POSITION){
                        next_data_write = RDS_WRITE_ENABLE;
                    }else if(segment_addr == RICOH5B_RDS_SEGMENT_ADDR_4 && write_pos == FOURTH_POSITION){
                        next_data_write = RDS_WRITE_ENABLE;
                    }
                }else{
                    ricoh5b_next_rds_eint(ricoh5b_dev);
                    return;
                }
            }
            
            if(rds_block == RICOH5B_RDS_BLOCK_C){
                if(next_data_write){
                    next_data_write = RDS_WRITE_DISABLE;
                    if((((RICOH5B_RDS_LIMIT_MIN <= buf_data_h[0]) && (buf_data_h[0] <= RICOH5B_RDS_LIMIT_MAX)) &&
                        ((RICOH5B_RDS_LIMIT_MIN <= buf_data_l[0]) && (buf_data_l[0] <= RICOH5B_RDS_LIMIT_MAX)))){
                            buf_rds_data[write_pos] = buf_data_h[0];
                            write_pos++;
                            buf_rds_data[write_pos] = buf_data_l[0];
                            write_pos++;
                    }else{
                        pr_err("%s: write data error h = 0x%x l = 0x%x \n",__func__, buf_data_h[0], buf_data_l[0]);
                        write_pos = FIRST_POSITION;
                        memset(&buf_rds_data, 0, sizeof(buf_rds_data));
                        ricoh5b_next_rds_eint(ricoh5b_dev);
                        return;
                    }
                    
                	if(write_pos == RICOH5B_RDS_BUF){
                	    write_pos = FIRST_POSITION;
                        for(i = 0; i < RICOH5B_RDS_BUF; i++){
                            if(buf_rds_data[i] == buf_rds_check[i]){
                                if(i == RICOH5B_LAST_DATA){
                                    mutex_lock(&ricoh5b_dev->repeat_check);
                                    memcpy(buf_rds_resdata, &buf_rds_check[0] , sizeof(buf_rds_resdata));
                                    #ifdef RICOH5B_MAIN_DEBUG
                                    pr_info("%s:buf_rds_resdata[0~7] = 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x 0x%x\n"
                                    , __func__, buf_rds_resdata[0], buf_rds_resdata[1],buf_rds_resdata[2]
                                    ,buf_rds_resdata[3],buf_rds_resdata[4],buf_rds_resdata[5],buf_rds_resdata[6]
                                    ,buf_rds_resdata[7]);
                                    #endif
                                    change_rds_data = POLL_CHECK_FLAG_ENABLE;
                                    mutex_unlock(&ricoh5b_dev->repeat_check);
                                    memset(&buf_rds_check, 0, sizeof(buf_rds_check));
                                    wake_up_interruptible(&ricoh5b_dev->read_wq);
                                }
                            }else{
                                memcpy(buf_rds_check, &buf_rds_data[0] , sizeof(buf_rds_check));
                                memset(&buf_rds_data, 0, sizeof(buf_rds_data));
                                break;
                            }
                        }
                    }
                }
            }
        }else{
            #ifdef RICOH5B_MAIN_DEBUG
            pr_info("%s: get rds data error \n",__func__);
            #endif
            write_pos = FIRST_POSITION;
            next_data_write = RDS_WRITE_DISABLE;
            memset(&buf_rds_data, 0, sizeof(buf_rds_data));
        }
    }
    
    ricoh5b_next_rds_eint(ricoh5b_dev);
	return;
}

static void ricoh5b_get_rds_irq(void)
{
    struct ricoh5b_dev *ricoh5b_dev;
    
    ricoh5b_dev = ricoh5b_device_info;

    queue_delayed_work(
        ricoh5b_dev->workqueue,
        &ricoh5b_dev->ricoh5b_rds_work,
        ACCESSARY_CHECK_DELAY_TIME
    );
	return;
}

static int ricoh5b_rds_set_status(struct ricoh5b_dev *ricoh5b_dev, int val)
{
    int ret = 0;
    
    mutex_lock(&ricoh5b_dev->rds_mode_mutex);
    if(val == RICOH5B_RDS_MODE_ON){

        ret = ricoh5b_cmd_write_bit( RICOH5B_REG_ADR_RDS_SETTING_1, rbds_enable, RICOH5B_REG_RDS_ENABLE);
        if(ret < 0){
            pr_err("%s:RICOH5B_REG_ADR_RDS_SETTING_1 err = %d \n",__func__, ret);
            goto send_err;
        }
        ret = ricoh5b_cmd_write_bit( RICOH5B_REG_ADR_RDS_SETTING_2, RICOH5B_REG_DAT_RDS_SETTING_2, 
                               RICOH5B_REG_RDS_ERR_VAL | RICOH5B_REG_RDS_SYNC_VAL);
        if(ret < 0){
            pr_err("%s:RICOH5B_REG_ADR_RDS_SETTING_2 err = %d \n",__func__, ret);
            goto send_err;
        }
        ricoh5b_dev->rds_mode = RICOH5B_RDS_MODE_ON;
        mt_eint_unmask(RICOH5B_RDSI_EINT);
    }else if(val == RICOH5B_RDS_MODE_OFF){
        unsigned char buf_status[READ_BUF_SIZE] = {0, };
        unsigned char buf_data_l[READ_BUF_SIZE] = {0, };
        unsigned char buf_data_h[READ_BUF_SIZE] = {0, };
        unsigned char buf_offset_l[READ_BUF_SIZE] = {0, };
        unsigned char buf_offset_h[READ_BUF_SIZE] = {0, };
        int gpio_val = 0;
        int i = 0;

        ret = ricoh5b_cmd_write_bit( RICOH5B_REG_ADR_RDS_SETTING_1, RICOH5B_REG_RDS_OFF_VALUE , RICOH5B_REG_RDS_ENABLE);
        if(ret < 0){
            pr_err("%s:RICOH5B_REG_ADR_RDS_SETTING_1 err = %d \n",__func__, ret);
            goto send_err;
        }
        ret = ricoh5b_cmd_write_bit( RICOH5B_REG_ADR_RDS_SETTING_2, RICOH5B_REG_DAT_RDS_SETTING_2, 
                               RICOH5B_REG_RDS_ERR_VAL | RICOH5B_REG_RDS_SYNC_VAL);
        if(ret < 0){
            pr_err("%s:RICOH5B_REG_ADR_RDS_SETTING_2 err = %d \n",__func__, ret);
            goto send_err;
        }
        ricoh5b_dev->rds_mode = RICOH5B_RDS_MODE_OFF;
        mt_eint_mask(RICOH5B_RDSI_EINT);
        
        for(i = 0; i < RICOH5B_RDS_RETRY; i++){
            gpio_val = mt_get_gpio_in(ricoh5b_dev->iic_gpio);
            if(gpio_val == 1){
                break;
            }

            ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_STATUS, buf_status);
            ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_DATA_L, buf_data_l);
            ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_DATA_H, buf_data_h);
            ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_OFFSET_L, buf_offset_l);
            ricoh5b_cmd_read_byte( RICOH5B_REG_ADR_RDS_OFFSET_H, buf_offset_h);

            msleep(10);
        }
        memset(&buf_rds_data, 0, sizeof(buf_rds_data));
        memset(&buf_rds_check, 0, sizeof(buf_rds_check));
	    memset(&buf_rds_resdata, 0, sizeof(buf_rds_resdata));
	    change_rds_data = POLL_CHECK_FLAG_DISABLE;
    }
    
    mutex_unlock(&ricoh5b_dev->rds_mode_mutex);

	return 0;
send_err:
    mutex_unlock(&ricoh5b_dev->rds_mode_mutex);
    return -1;
}

static int ricoh5b_read(struct file *file, void *buf, size_t len)
{
	struct ricoh5b_dev *ricoh5b_dev;
	char res_buf[9] = {0};
	int ret = 0;
	int res_size = 0;

	ricoh5b_dev = ricoh5b_device_info;
	
    mutex_lock(&ricoh5b_dev->repeat_check);
	res_size = sizeof(res_buf);

	if(change_signal_strength == POLL_CHECK_FLAG_ENABLE){
	    res_buf[0] = SIGNAL_STRENGTH_FLAG;
	    res_buf[1] = res_signal_strength;
	}else if(change_audmode == POLL_CHECK_FLAG_ENABLE){
	    res_buf[0] = REC_AUDIO_MODE_FLAG;
	    res_buf[1] = ricoh5b_dev->rec_audio_mode;
	}else if(change_rds_data == POLL_CHECK_FLAG_ENABLE){
	    res_buf[0] = RDS_FLAG;
	    memcpy(&res_buf[1], &buf_rds_resdata[0] , RICOH5B_RDS_BUF);
	}

    ret = copy_to_user((void *)buf, res_buf, res_size);
    if(ret != 0){
	    pr_err("%s:copy_to_user Error ret = %d \n", __func__, ret);
	    mutex_unlock(&ricoh5b_dev->repeat_check);
	    return -1;
    }else{
        if(res_buf[0] == SIGNAL_STRENGTH_FLAG){
            change_signal_strength = POLL_CHECK_FLAG_DISABLE;
        }else if(res_buf[0] == REC_AUDIO_MODE_FLAG){
            change_audmode = POLL_CHECK_FLAG_DISABLE;
        }else if(res_buf[0] == RDS_FLAG){
            change_rds_data = POLL_CHECK_FLAG_DISABLE;
            memset(&buf_rds_resdata, 0, sizeof(buf_rds_resdata));
        }
    }
	
	mutex_unlock(&ricoh5b_dev->repeat_check);
	
	return res_size;
}

static int ricoh5b_open(struct file *file)
{
	struct ricoh5b_dev *ricoh5b_dev;
	int ret = 0;

	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s: Start\n",__func__);
	#endif

    ricoh5b_dev = ricoh5b_device_info;

    /* Tuner VDD(5V) On */
   	ret = mt_set_gpio_out(ricoh5b_dev->vdd_gpio, 1);
	if(ret){
		pr_err("%s : ricoh5b_dev->vdd_gpio : ret= %d\n", __func__, ret);
	}

	msleep(20);

	/* Tuner Power On */
	ret = mt_set_gpio_out(ricoh5b_dev->ce_gpio, 1);
	if(ret){
		pr_err("%s : ricoh5b_dev->ce_gpio : ret= %d\n", __func__, ret);
	}	

	msleep(20);

	mutex_lock(&ricoh5b_dev->read_mutex);
	ret = ricoh5b_initialize_setting();
	if(ret < 0){
	    pr_err("%s:ricoh5b_initialize_setting err = %d \n",__func__, ret);
	    goto send_err;
	}

	ret = ricoh5b_audio_output_mute_on();
	if(ret < 0){
	    pr_err("%s:ricoh5b_audio_output_mute_on err = %d \n",__func__, ret);
	    goto send_err;
	}else{
	    ricoh5b_dev->mute_state = RICOH5B_MUTE_ON;
	}
	
	ret = ricoh5b_send_audio_output_setting(RICOH5B_SELECT_AUDIO_OUTPUT);
	if(ret < 0){
	    pr_err("%s:ricoh5b_send_audio_output_setting err = %d \n",__func__, ret);
	    goto send_err;
	}

	ret = ricoh5b_fm_send_set_band(ricoh5b_dev->tuned_freq / 10, gain_value);
	if(ret < 0){
        pr_err("%s:ricoh5b_fm_send_set_band err = %d \n",__func__, ret);
	    goto send_err;
	}

	/* deemphasis set */
    ret = ricoh5b_send_set_de_emphasis(ricoh5b_dev);
   	if(ret < 0){
        pr_err("%s:ricoh5b_send_set_de_emphasis err = %d \n",__func__, ret);
	    goto send_err;
	}

	mutex_unlock(&ricoh5b_dev->read_mutex);

	memset(&buf_rds_data, 0, sizeof(buf_rds_data));
	memset(&buf_rds_check, 0, sizeof(buf_rds_check));
	memset(&buf_rds_resdata, 0, sizeof(buf_rds_resdata));

	tuner_open = 1;
	change_rds_data = POLL_CHECK_FLAG_DISABLE;

    queue_delayed_work(
        workqueue_check,
        &ricoh5b_check_work,
        RICOH5B_STATE_POLL_TIME
    );
    
	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : End\n", __func__);
	#endif
	
    return 0;
send_err:
    mutex_unlock(&ricoh5b_dev->read_mutex);
	return -EIO;
}

static int ricoh5b_close(struct file *file)
{
    struct ricoh5b_dev *ricoh5b_dev;
    int ret = 0;

    #ifdef RICOH5B_MAIN_DEBUG
    pr_info("%s: Start\n",__func__);
    #endif

    ricoh5b_dev = ricoh5b_device_info;
    
    tuner_open = 0;

    cancel_delayed_work(&ricoh5b_check_work);
    flush_workqueue(workqueue_check);
    
    ret = ricoh5b_rds_set_status(ricoh5b_dev, 0);
    if(ret < 0){
        pr_err("%s: rds stop Error \n",__func__);
    }

	cancel_delayed_work(&ricoh5b_dev->ricoh5b_rds_work);
    flush_workqueue(ricoh5b_dev->workqueue);

    mutex_lock(&ricoh5b_dev->read_mutex);

    ret = ricoh5b_power_stop();
    if(ret < 0){
        pr_err("%s:ricoh5b_power_stop err = %d \n",__func__, ret);
    }
    mutex_unlock(&ricoh5b_dev->read_mutex);
    /* Vdd Power off */
    ret = mt_set_gpio_out(ricoh5b_dev->vdd_gpio, 0);
    if(ret){
	    pr_err("%s : ricoh5b_dev->vdd_gpio : ret= %d\n", __func__, ret);
    }
	
	/* Power off */
	ret = mt_set_gpio_out(ricoh5b_dev->ce_gpio, 0);
	if(ret){
		pr_err("%s : ricoh5b_dev->ce_gpio : ret= %d\n", __func__, ret);
	}

    #ifdef RICOH5B_MAIN_DEBUG
    pr_info("%s : End\n", __func__);
    #endif
    return 0;
}

static int ricoh5b_poll(struct file *file, poll_table * wait)
{
	struct ricoh5b_dev *ricoh5b_dev;
	int ret = 0;

    ricoh5b_dev = ricoh5b_device_info;

	poll_wait(file, &ricoh5b_dev->read_wq, wait);

	mutex_lock(&ricoh5b_dev->repeat_check);
	
    if(change_rds_data == POLL_CHECK_FLAG_ENABLE || 
       change_signal_strength == POLL_CHECK_FLAG_ENABLE ||
       change_audmode == POLL_CHECK_FLAG_ENABLE){
		ret |= POLLIN;
	}else{
		ret = 0;
	}
	mutex_unlock(&ricoh5b_dev->repeat_check);
	
	return ret;
}

static int ricoh5b_querycap( struct file *file, void *dummy, struct v4l2_capability *capability)
{
	strlcpy(capability->driver, "ricoh5b tuner dirver", sizeof(capability->driver));
	strlcpy(capability->card, "ricoh5b", sizeof(capability->card));
	strlcpy(capability->bus_info, "i2c-1", sizeof(capability->card));
	capability->version	= KERNEL_VERSION(1, 0, 0);
	capability->capabilities = V4L2_CAP_TUNER | V4L2_CAP_RADIO;

	return 0;
}

static int ricoh5b_s_tuner(struct file *file, void *dummy, struct v4l2_tuner *tuner)
{
	int ret = 0;
	unsigned char val = 0;
	__u32 set_mode = 0;
	struct ricoh5b_dev *ricoh5b_dev;

	ricoh5b_dev = ricoh5b_device_info;
	
	if(tuner->audmode == V4L2_TUNER_MODE_STEREO){
		val = RICOH5B_REG_DAT_MONO_OFF;
		set_mode = V4L2_TUNER_MODE_STEREO;
	}else if(tuner->audmode == V4L2_TUNER_MODE_MONO){
		val = RICOH5B_REG_DAT_MONO_ON;
		set_mode = V4L2_TUNER_MODE_MONO;
	}

	ret = ricoh5b_cmd_write_bit( RICOH5B_REG_ADR_MONO, val, RICOH5B_REG_BIT_MONO);
	
	if(ret != RICOH5B_REG_ADDRESS_LENGTH + DATA_1BYTE_LENGTH){
		pr_err("%s:ricoh5b_cmd_write_bit Error = %d \n", __func__, ret);
		return ret;
	}
	
	ricoh5b_dev->audio_mode = set_mode;
	
	return 0;
}

static int ricoh5b_g_tuner(struct file *file, void *dummy, struct v4l2_tuner *tuner)
{
	struct ricoh5b_dev *ricoh5b_dev;
	int ret = 0;

	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : Start\n", __func__);
	#endif

    ricoh5b_dev = ricoh5b_device_info;
	
	strlcpy(tuner->name,"FM",sizeof(tuner->name));
	
	tuner->type = V4L2_TUNER_RADIO;
	tuner->capability = V4L2_TUNER_CAP_LOW | V4L2_TUNER_CAP_STEREO;
	
	if(ricoh5b_dev->audio_mode == V4L2_TUNER_MODE_STEREO){
		tuner->audmode = V4L2_TUNER_MODE_STEREO;
	}else if(ricoh5b_dev->audio_mode == V4L2_TUNER_MODE_MONO){
		tuner->audmode = V4L2_TUNER_MODE_MONO;
	}
	
	ret = ricoh5b_station_check(ricoh5b_dev);
    mutex_lock(&ricoh5b_dev->repeat_check);
	if(ret < 0){
	    tuner->signal = 0;
	    res_signal_strength = 0;
	}else{
	    tuner->signal = 1;
	    res_signal_strength = 1;
	}
    mutex_unlock(&ricoh5b_dev->repeat_check);
	
	/* not support */
	tuner->afc = 0;
	
	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : End\n", __func__);
	#endif

	return 0;
}

static int ricoh5b_s_frequency(struct file *file, void *dummy, struct v4l2_frequency *frequency)
{
	struct ricoh5b_dev *ricoh5b_dev;
	int ret = 0;
    unsigned short freq_set_val = 0;
    unsigned int  freq          = 0;
    
   	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : Start\n", __func__);
	#endif

	ricoh5b_dev = ricoh5b_device_info;
	
	freq = frequency->frequency >> 4;
	
	freq_set_val = (unsigned short)(freq / 10);

    ret = ricoh5b_fm_send_set_freq(freq_set_val, 0);
    if(ret < 0){
		pr_err("%s: ricoh5b_fm_send_set_freq err ret = %d \n",__func__, ret);
		return -1;
	}

	ricoh5b_dev->tuned_freq = freq;
	
	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : End\n", __func__);
	#endif
	return 0;
}

static int ricoh5b_g_frequency( struct file *file, void *dummy, struct v4l2_frequency *frequency)
{
	struct ricoh5b_dev *ricoh5b_dev;

	ricoh5b_dev = ricoh5b_device_info;
	
	if(frequency->tuner != 0){
		printk(KERN_ERR "%s(): invalid tuner %d\n",__FUNCTION__,frequency->tuner);
		return -EINVAL;
	}

	frequency->type = V4L2_TUNER_RADIO;
	frequency->frequency = (unsigned int)ricoh5b_dev->tuned_freq * 16;

	return 0;
}

static int ricoh5b_sense_mode_set(struct ricoh5b_dev *ricoh5b_dev, int value)
{
	int ret = 0;
	
	ret = ricoh5b_threshold_set( ricoh5b_dev, value);
	
	if(ret < 0){
		pr_err("%s: write bit Error ret = %d \n",__func__, ret);
		return -1;
	}
	
	ricoh5b_dev->sense_mode = value;

	return 0;
}

/* threshold value set */
static int ricoh5b_threshold_set(struct ricoh5b_dev *ricoh5b_dev, int value)
{
	int ret = 0;
	unsigned char val = 0;

	if(value == SENSE_MODE_HIGH){
    	val = ricoh5b_dev->sense_high;
	}else if(value == SENSE_MODE_LOW){
    	val = ricoh5b_dev->sense_low;
	}

	ret = ricoh5b_cmd_write_bit( RICOH5B_REG_ADR_RSSI_THR, val, RICOH5B_REG_BIT_RSSI_THR);
	
	if(ret != RICOH5B_REG_ADDRESS_LENGTH + DATA_1BYTE_LENGTH){
		pr_err("%s: write bit Error ret = %d \n",__func__, ret);
		return -1;
	}
	
	return 0;
}

static int ricoh5b_gainvalue_set(int gain)
{
    int ret = 0;
    unsigned char volume_data[RICOH5B_VOLUME_DATA_SIZE];
    unsigned char fm_volume = 0;

    switch(gain){
    case GAIN_0DB:
        gain_value = RICOH5B_VOLUME_0DB;
        fm_volume = RICOH5B_REG_DAT_HPVOL_FM_0DB;
        break;
    case GAIN_MINUS3DB:
        gain_value = RICOH5B_VOLUME_MINUS_3DB;
        fm_volume = RICOH5B_REG_DAT_HPVOL_FM_MINUS_3DB;
        break;
    case GAIN_MINUS4DB:
        gain_value = RICOH5B_VOLUME_MINUS_4DB;
        fm_volume = RICOH5B_REG_DAT_HPVOL_FM_MINUS_4DB;
        break;
    case GAIN_MINUS5DB:
        gain_value = RICOH5B_VOLUME_MINUS_5DB;
        fm_volume = RICOH5B_REG_DAT_HPVOL_FM_MINUS_5DB;
        break;
    default:
        gain_value = RICOH5B_VOLUME_0DB;
        fm_volume = RICOH5B_REG_DAT_HPVOL_FM_0DB;
        break;
    }

    volume_data[0] = fm_volume;
    volume_data[1] = fm_volume;
    ret = ricoh5b_cmd_write_byte(RICOH5B_REG_ADR_HPLVOL, &volume_data[0], RICOH5B_REG_ADDRESS_LENGTH + RICOH5B_REG_LENGTH_VOL);
    if(ret < 0){
        pr_err("%s:BAND_STEP_VOLUME err = %d \n",__func__, ret);
        goto send_err;
    }

    return 0;

send_err:
    return -1;
}

static void ricoh5b_rdsmode_set(int mode)
{
   	if(mode == RDS_MODE_RDS){
        rbds_enable = RICOH5B_REG_RDS_ON_VALUE;
    }else if(mode == RDS_MODE_RBDS){
        rbds_enable = RICOH5B_REG_RDS_ON_VALUE | RICOH5B_REG_DAT_RDS_SETTING_1_RBDS;
    }else{
        rbds_enable = RICOH5B_REG_RDS_ON_VALUE;
    }

    return;
}

static int ricoh5b_emphasis_set(struct ricoh5b_dev *ricoh5b_dev, int emphasis)
{
    int ret = 0;

    switch(emphasis){
    case DEEMPHASIS_50US:
        ricoh5b_dev->deemp    = DEEMPHASIS_DEMP_HIGH;
        ricoh5b_dev->deemp_ce = DEEMPHASIS_LOW;
        break;
    case DEEMPHASIS_60US:
        ricoh5b_dev->deemp    = DEEMPHASIS_DEMP_HIGH;
        ricoh5b_dev->deemp_ce = DEEMPHASIS_DEMP_CE_HIGH;
        break;
    case DEEMPHASIS_75US:
        ricoh5b_dev->deemp    = DEEMPHASIS_LOW;
        ricoh5b_dev->deemp_ce = DEEMPHASIS_LOW;
        break;
    case DEEMPHASIS_100US:
        ricoh5b_dev->deemp    = DEEMPHASIS_LOW;
        ricoh5b_dev->deemp_ce = DEEMPHASIS_DEMP_CE_HIGH;
        break;
    default:
        ricoh5b_dev->deemp    = DEEMPHASIS_DEMP_HIGH;
        ricoh5b_dev->deemp_ce = DEEMPHASIS_LOW;
        break;
    }

    /* deemphasis set */
    ret = ricoh5b_send_set_de_emphasis(ricoh5b_dev);
   	if(ret < 0){
        pr_err("%s:ricoh5b_send_set_de_emphasis err = %d \n",__func__, ret);
	    goto send_err;
	}

    return 0;

send_err:
    return -1;
}

static int ricoh5b_s_ctrl( struct file *file, void *dummy, struct v4l2_control *control)
{
	struct ricoh5b_dev *ricoh5b_dev;
	int ret = 0;

	ricoh5b_dev = ricoh5b_device_info;
	
	switch(control->id){
	case V4L2_CID_AUDIO_MUTE:
		if(control->value){
			/* mute on */
			ret = ricoh5b_audio_output_mute_on();
			if(ret < 0){
			    pr_err("%s: ricoh5b_audio_output_mute_on Error\n", __func__);
			}else{
			    ricoh5b_dev->mute_state = RICOH5B_MUTE_ON;
			}
		}else{
			/* mute off */
			ret = ricoh5b_audio_output_mute_off();
			if(ret < 0){
			    pr_err("%s: ricoh5b_audio_output_mute_off Error\n", __func__);
			}else{
			    ricoh5b_dev->mute_state = RICOH5B_MUTE_OFF;
			}
		}
		break;
	case V4L2_CID_SENSE_MODE:
		ret = ricoh5b_sense_mode_set(ricoh5b_dev, control->value);
		if(ret < 0){
		    pr_err("%s: ricoh5b_sense_mode_set Error\n", __func__);
        }
		break;
	case V4L2_CID_SENSE_THRESHOLD_HIGH:
		ricoh5b_dev->sense_high=(unsigned char)(control->value&0xFF);
		ret = ricoh5b_threshold_set(ricoh5b_dev, ricoh5b_dev->sense_mode);
		if(ret < 0){
		    pr_err("%s: ricoh5b_threshold_set Error\n",__func__);
		}
		break;
	 	case V4L2_CID_SENSE_THRESHOLD_LOW:
		ricoh5b_dev->sense_low=(unsigned char)(control->value&0xFF);
		ret = ricoh5b_threshold_set(ricoh5b_dev, ricoh5b_dev->sense_mode);
		if(ret < 0){
		    pr_err("%s: ricoh5b_threshold_set Error\n",__func__);
		}
		break;
	case V4L2_CID_RDS_ENABLE:
		ret = ricoh5b_rds_set_status( ricoh5b_dev, control->value);
		if(ret < 0){
	        pr_err("%s: ricoh5b_rds_set_status Error\n",__func__);
	    }
		break;
	case V4L2_CID_GAIN_SETTING:
	    ret = ricoh5b_gainvalue_set(control->value);
	    if(ret < 0){
	        pr_err("%s: ricoh5b_gainvalue_set Error\n",__func__);
	    }
	    break;
	case V4L2_CID_RDS_MODE:
	    ricoh5b_rdsmode_set(control->value);
	    break;
	case V4L2_CID_DEEMPHASIS:
	    ret = ricoh5b_emphasis_set(ricoh5b_dev, control->value);
	    if(ret < 0){
	        pr_err("%s: ricoh5b_emphasis_set Error\n",__func__);
	    }
	    break;
	case V4L2_CID_READ_REGISTER:
	{
		int ret = 0;
		int value = 0;

		ret = ricoh5b_read_reg(control->value, &value);
		if (ret < 0) {
			pr_err("%s:ricoh5b_read_reg Error\n", __func__);
			return -1;
		}
		control->value = value;
		break;
	}
	default:
		return(-EINVAL);
	}

	return ret;
}

static int ricoh5b_g_ctrl( struct file *file, void *dummy, struct v4l2_control *control)
{
	struct ricoh5b_dev *ricoh5b_dev;

	ricoh5b_dev = ricoh5b_device_info;
	
	switch(control->id){
	case V4L2_CID_AUDIO_MUTE:
		control->value = ricoh5b_dev->mute_state;
		break;
	case V4L2_CID_SENSE_MODE:
		control->value = ricoh5b_dev->sense_mode;
		break;
	case V4L2_CID_SENSE_THRESHOLD_HIGH:
		control->value = ricoh5b_dev->sense_high;
		break;
	case V4L2_CID_SENSE_THRESHOLD_LOW:
		control->value = ricoh5b_dev->sense_low;
		break;
	case V4L2_CID_AUDIO_MODE:
	{
        int ret = 0;
        mutex_lock(&ricoh5b_dev->repeat_check);
	    ret = ricoh5b_rec_audio_mode();
		if(ret < 0){
		    pr_err("%s:rec_audio_mode err ret = %d \n", __func__, ret);
		    mutex_unlock(&ricoh5b_dev->repeat_check);
		    return -1;
		}
		ricoh5b_dev->rec_audio_mode = ret;
		control->value = ricoh5b_dev->rec_audio_mode;
		mutex_unlock(&ricoh5b_dev->repeat_check);
		break;
	}
	case V4L2_CID_RAW_SIGNAL_STRENGTH:
	{
	    int rssi_raw = 0;
	    int ret = 0;
	    ret = ricoh5b_rssi_check(RICOH5B_RSSI_THRESHOLD, &rssi_raw);
	    if(ret < 0){
	        pr_err("%s:ricoh5b_rssi_check Error\n",__func__);
	        control->value = 0;
	        return -1;
	    }
	    control->value = rssi_raw;
	    break;
	}
	default:
		return(-EINVAL);
	}

	return 0;
}

static int ricoh5b_read_reg(int address, int *value)
{
	int ret = 0;
	unsigned char buf_r[READ_BUF_SIZE] = {0, };

	ret = ricoh5b_cmd_read_byte((unsigned char)address, buf_r);

	*value = buf_r[0];

	return ret;
}

static int ricoh5b_codec_probe(struct snd_soc_codec *codec)
{
	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : Start\n", __func__);
	#endif
	return 0;
}

static int ricoh5b_codec_remove(struct snd_soc_codec *codec)
{
	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : Start\n", __func__);
	#endif
	return 0;
}

static struct snd_soc_dai_driver ricoh5b_dai = {
	.name = "ricoh5b-i2s",
	.capture = {
		.stream_name = "ricoh5b_cap",
		.channels_min = 1,
		.channels_max = 2,
		.rates = SNDRV_PCM_RATE_8000_192000,
		.formats = (SNDRV_PCM_FMTBIT_S16_LE | SNDRV_PCM_FMTBIT_S32_LE)},
};

static struct snd_soc_codec_driver soc_codec_dev_ricoh5b = {
	.probe  = ricoh5b_codec_probe,
	.remove = ricoh5b_codec_remove,
};

static int ricoh5b_probe(struct i2c_client *client,
           const struct i2c_device_id *id)
{
	int ret = 0;
	struct ricoh5b_dev *ricoh5b_dev = NULL;
	RICOH5B_PLATFORM_RESOURCE *resource = NULL;

	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : Start\n", __func__);
	#endif

/* SAKANA 20160310 KAMIJO [start] */
#if 0
	mt_set_gpio_pull_enable(RICOH5B_I2S5_BCK_GPIO, GPIO_PULL_DISABLE);
	mt_set_gpio_pull_enable(RICOH5B_I2S5_LRCK_GPIO, GPIO_PULL_DISABLE);
	mt_set_gpio_pull_enable(RICOH5B_I2S5_DATA_GPIO, GPIO_PULL_DISABLE);
	mt_set_gpio_mode(RICOH5B_I2S5_BCK_GPIO, GPIO_MODE_01);
	mt_set_gpio_mode(RICOH5B_I2S5_LRCK_GPIO, GPIO_MODE_01);
	mt_set_gpio_mode(RICOH5B_I2S5_DATA_GPIO, GPIO_MODE_01);
#else
	if (RICOH5B_SELECT_AUDIO_OUTPUT !=  RICOH5B_AUDIO_OUTPUT_ANALOG) {
		mt_set_gpio_pull_enable(RICOH5B_I2S5_BCK_GPIO, GPIO_PULL_DISABLE);
		mt_set_gpio_pull_enable(RICOH5B_I2S5_LRCK_GPIO, GPIO_PULL_DISABLE);
		mt_set_gpio_pull_enable(RICOH5B_I2S5_DATA_GPIO, GPIO_PULL_DISABLE);
		mt_set_gpio_mode(RICOH5B_I2S5_BCK_GPIO, GPIO_MODE_01);
		mt_set_gpio_mode(RICOH5B_I2S5_LRCK_GPIO, GPIO_MODE_01);
		mt_set_gpio_mode(RICOH5B_I2S5_DATA_GPIO, GPIO_MODE_01);
	} else {
		/* Not use I2S */
	}
#endif
/* SAKANA 20150310 KAMIJO [end] */
	
	resource = client->dev.platform_data;	

	ricoh5b_dev = kzalloc(sizeof(*ricoh5b_dev), GFP_KERNEL);
	if (ricoh5b_dev == NULL) {
		dev_err(&client->dev,
			"failed to allocate memory for module data\n");
		ret = -ENOMEM;
		goto err_exit;
	}

    ricoh5b_dev->ce_gpio  = resource->ce;
    ricoh5b_dev->iic_gpio = resource->i2c_rds;
   	ricoh5b_dev->client	  = client;
    ricoh5b_dev->vdd_gpio = resource->vdd;

    mt_set_gpio_pull_enable(ricoh5b_dev->iic_gpio, GPIO_PULL_DISABLE);
    mt_set_gpio_pull_select(ricoh5b_dev->iic_gpio, GPIO_PULL_UP);
    mt_set_gpio_dir(ricoh5b_dev->iic_gpio, GPIO_DIR_IN);
    mt_set_gpio_mode(ricoh5b_dev->iic_gpio, GPIO_MODE_GPIO);
    msleep(20);

	mt_set_gpio_pull_select(ricoh5b_dev->ce_gpio, GPIO_PULL_DOWN);
	mt_set_gpio_pull_enable(ricoh5b_dev->ce_gpio, GPIO_PULL_ENABLE);
	mt_set_gpio_out(ricoh5b_dev->ce_gpio, 0);
	mt_set_gpio_dir(ricoh5b_dev->ce_gpio, GPIO_DIR_OUT);
	mt_set_gpio_mode(ricoh5b_dev->ce_gpio, GPIO_MODE_GPIO);
	msleep(20);
	
    mt_set_gpio_pull_select(ricoh5b_dev->vdd_gpio, GPIO_PULL_DOWN);
    mt_set_gpio_pull_enable(ricoh5b_dev->vdd_gpio, GPIO_PULL_ENABLE);
   	mt_set_gpio_out(ricoh5b_dev->vdd_gpio, 0);
    mt_set_gpio_dir(ricoh5b_dev->vdd_gpio, GPIO_DIR_OUT);
    mt_set_gpio_mode(ricoh5b_dev->vdd_gpio, GPIO_MODE_GPIO);
    msleep(20);

    /* init mutex and queues */
	init_waitqueue_head(&ricoh5b_dev->read_wq);
	mutex_init(&ricoh5b_dev->read_mutex);
	mutex_init(&ricoh5b_dev->station_mutex);
	mutex_init(&ricoh5b_dev->rds_mode_mutex);
	mutex_init(&ricoh5b_dev->repeat_check);
	
	ricoh5b_dev->workqueue = create_singlethread_workqueue("ricoh5b_rds");
	if(!ricoh5b_dev->workqueue){
			pr_err("%s: create_singlethread_workqueue() rds error\n", __func__);
			goto err_workqueue;
	}
	
	workqueue_check = create_singlethread_workqueue("ricoh5b_check");
	if(!workqueue_check){
			pr_err("%s: create_singlethread_workqueue() workqueue_check error\n", __func__);
			goto err_workqueue;
	}

	INIT_DELAYED_WORK(&ricoh5b_dev->ricoh5b_rds_work, ricoh5b_get_rds);
	INIT_DELAYED_WORK(&ricoh5b_check_work, ricoh5b_repeat_check);
	
	i2c_set_clientdata(client, ricoh5b_dev);

/* SAKANA 20160310 KAMIJO [start] */
#if 0
	ret = snd_soc_register_codec(&client->dev, &soc_codec_dev_ricoh5b, &ricoh5b_dai, 1);
#else
	if (RICOH5B_SELECT_AUDIO_OUTPUT !=  RICOH5B_AUDIO_OUTPUT_ANALOG) {
		ret = snd_soc_register_codec(&client->dev, &soc_codec_dev_ricoh5b, &ricoh5b_dai, 1);
	} else {
		/* Not use I2S */
	}
#endif
/* SAKANA 20150310 KAMIJO [end] */

	if(ret < 0){
	    pr_err("%s: snd_soc_register_codec error = %d \n", __func__, ret);
	    goto err_snd_soc_register;
	}

	ricoh5b_dev->video_device = video_device_alloc();
	
	if(ricoh5b_dev->video_device == NULL){
		 pr_err("%s: ricoh5b_dev->video_device NULL\n",__func__);
		 goto err_video_device_register;
	}

	strncpy(ricoh5b_dev->video_device->name, "ricoh5b_Tuner", sizeof(ricoh5b_dev->video_device->name));
	
	ricoh5b_dev->video_device->release	 = video_device_release_empty;
	ricoh5b_dev->video_device->fops		 = &ricoh5b_file_operations;
	ricoh5b_dev->video_device->ioctl_ops = &ricoh5b_ioctl_operations;

	/* set private data to video structure */
	video_set_drvdata(ricoh5b_dev->video_device,ricoh5b_dev);

	ret = video_register_device(ricoh5b_dev->video_device,VFL_TYPE_RADIO,-1);
	if(ret < 0) {
		pr_err("%s(): video_register_device() error\n",__func__);
		goto err_video_register;
	}
	
	/* init status value set */
	ricoh5b_dev->tuned_freq = RICOH5B_FIRST_FREQ;
	ricoh5b_dev->audio_mode = RICOH5B_MODE_STEREO;
	ricoh5b_dev->mute_state = RICOH5B_MUTE_ON;
	ricoh5b_dev->sense_mode = RICOH5B_FIRST_SENSE_MODE;
	ricoh5b_dev->sense_high = RICOH5B_FIRST_SENSE_HIGH;
	ricoh5b_dev->sense_low	= RICOH5B_FIRST_SENSE_LOW;
	ricoh5b_dev->rds_mode	= RICOH5B_RDS_MODE_OFF;
	ricoh5b_dev->rec_audio_mode = RICOH5B_MODE_MONO;
	ricoh5b_dev->station_check = 0;

	if(DeEmphasis == DEEMPHASIS_50US){
	    ricoh5b_dev->deemp    = DEEMPHASIS_DEMP_HIGH;
		ricoh5b_dev->deemp_ce = DEEMPHASIS_LOW;
	}else if(DeEmphasis == DEEMPHASIS_60US){
		ricoh5b_dev->deemp    = DEEMPHASIS_DEMP_HIGH;
		ricoh5b_dev->deemp_ce = DEEMPHASIS_DEMP_CE_HIGH;
	}else if(DeEmphasis == DEEMPHASIS_75US){
		ricoh5b_dev->deemp    = DEEMPHASIS_LOW;
		ricoh5b_dev->deemp_ce = DEEMPHASIS_LOW;
	}else if(DeEmphasis == DEEMPHASIS_100US){
		ricoh5b_dev->deemp    = DEEMPHASIS_LOW;
		ricoh5b_dev->deemp_ce = DEEMPHASIS_DEMP_CE_HIGH;
	}else{
	    ricoh5b_dev->deemp    = DEEMPHASIS_DEMP_HIGH;
		ricoh5b_dev->deemp_ce = DEEMPHASIS_LOW;
	}

	if(GainValue == GAIN_0DB){
	    gain_value = RICOH5B_VOLUME_0DB;
	}else if(GainValue == GAIN_MINUS3DB){
	    gain_value = RICOH5B_VOLUME_MINUS_3DB;
	}else if(GainValue == GAIN_MINUS4DB){
	    gain_value = RICOH5B_VOLUME_MINUS_4DB;
	}else if(GainValue == GAIN_MINUS5DB){
	    gain_value = RICOH5B_VOLUME_MINUS_5DB;
	}else{
	    gain_value = RICOH5B_VOLUME_0DB;
	}

	if(RdsMode == RDS_MODE_RDS){
	    rbds_enable = RICOH5B_REG_RDS_ON_VALUE;
	}else if(RdsMode == RDS_MODE_RBDS){
	    rbds_enable = RICOH5B_REG_RDS_ON_VALUE | RICOH5B_REG_DAT_RDS_SETTING_1_RBDS;
	}else{
	    rbds_enable = RICOH5B_REG_RDS_ON_VALUE;
	}

    ricoh5b_i2c_client = ricoh5b_dev->client;
	ricoh5b_device_info = ricoh5b_dev;
	
	/* pin interrupt setting */
    mt_eint_registration( RICOH5B_RDSI_EINT,
                         EINTF_TRIGGER_FALLING,
                         ricoh5b_get_rds_irq,
                         0);

    mt_eint_dis_debounce(RICOH5B_RDSI_EINT);
    mt_eint_mask(RICOH5B_RDSI_EINT);

	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : End \n", __func__);
	#endif

#ifdef CONFIG_REGMON_DEBUG	
	ricoh5b_regmon_add(ricoh5b_dev);
#endif

	return 0;

err_video_register:
	video_device_release(ricoh5b_dev->video_device);
err_workqueue:
err_snd_soc_register:
err_video_device_register:
	mutex_destroy(&ricoh5b_dev->read_mutex);
	mutex_destroy(&ricoh5b_dev->station_mutex);
	mutex_destroy(&ricoh5b_dev->rds_mode_mutex);
	mutex_destroy(&ricoh5b_dev->repeat_check);
	kfree(ricoh5b_dev);
	ricoh5b_dev = NULL;
err_exit:
	gpio_free(resource->ce);
	gpio_free(resource->i2c_rds);

	return ret;
}

static int ricoh5b_remove(struct i2c_client *client)
{
	struct ricoh5b_dev *ricoh5b_dev;
	int ret = 0;

	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : Start\n", __func__);
	#endif

	ricoh5b_dev = i2c_get_clientdata(client);
	
	mt_eint_mask(RICOH5B_RDSI_EINT);
	
	cancel_delayed_work(&ricoh5b_check_work);
    flush_workqueue(workqueue_check);
    destroy_workqueue(workqueue_check);

	cancel_delayed_work(&ricoh5b_dev->ricoh5b_rds_work);
    flush_workqueue(ricoh5b_dev->workqueue);
    destroy_workqueue(ricoh5b_dev->workqueue);

    if(tuner_open){
        /* ricoh5b off Sequence */
        ret = ricoh5b_power_stop();
        if(ret){
            pr_err("%s : ricoh5b_power_stop : ret= %d\n", __func__, ret);
        }
    }

    ret = mt_set_gpio_out(ricoh5b_dev->vdd_gpio, 0);
    if(ret){
        pr_err("%s : RICOH5B_EINT6_GPIO : ret= %d\n", __func__, ret);
    }

    /* Power off */
    ret = mt_set_gpio_out(ricoh5b_dev->ce_gpio, 0);
    if(ret){
	    pr_err("%s : ricoh5b_dev->ce_gpio : ret= %d\n", __func__, ret);
    }

    msleep(20);

    snd_soc_unregister_codec(&client->dev);
	video_unregister_device(ricoh5b_dev->video_device);
	video_device_release(ricoh5b_dev->video_device);
	mutex_destroy(&ricoh5b_dev->read_mutex);
	mutex_destroy(&ricoh5b_dev->station_mutex);
	mutex_destroy(&ricoh5b_dev->rds_mode_mutex);
	mutex_destroy(&ricoh5b_dev->repeat_check);
	
	kfree(ricoh5b_dev);
	ricoh5b_dev = NULL;
	
    #ifdef CONFIG_REGMON_DEBUG
	ricoh5b_regmon_del();
    #endif

    #ifdef RICOH5B_MAIN_DEBUG
    pr_info("%s: End\n", __func__);
    #endif

	return ret;
}

static int ricoh5b_suspend(struct i2c_client *client, pm_message_t message)
{
	int ret = 0;
	struct ricoh5b_dev *ricoh5b_dev;

	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : Start\n", __func__);
	#endif

    ricoh5b_i2c_client  = client;
    ricoh5b_dev = ricoh5b_device_info;

	if(tuner_open){
        mt_eint_mask(RICOH5B_RDSI_EINT);

        cancel_delayed_work(&ricoh5b_check_work);
        flush_workqueue(workqueue_check);
	    cancel_delayed_work(&ricoh5b_dev->ricoh5b_rds_work);
        flush_workqueue(ricoh5b_dev->workqueue);

        ret = mt_set_gpio_out(ricoh5b_dev->vdd_gpio, 1);
        if(ret){
            pr_err("%s : ricoh5b_dev->vdd_gpio : ret= %d\n", __func__, ret);
        }
	
	    msleep(20);

        /* ricoh5b off Sequence */
        ret = ricoh5b_power_stop();
        if(ret < 0){
            pr_err("%s:RICOH5B_REG_ADR_RDS_SETTING_2 err = %d \n",__func__, ret);
        }
    }

    ret = mt_set_gpio_out(ricoh5b_dev->vdd_gpio, 0);
    if(ret){
        pr_err("%s : ricoh5b_dev->vdd_gpio : ret= %d\n", __func__, ret);
    }

	/* Power off */
	ret = mt_set_gpio_out(ricoh5b_dev->ce_gpio, 0);
	if(ret){
	    pr_err("%s : ricoh5b_dev->ce_gpio : ret= %d\n", __func__, ret);
	}

    #ifdef RICOH5B_MAIN_DEBUG
    pr_info("%s :End \n", __func__);
	#endif

	return ret;
}

static int ricoh5b_resume(struct i2c_client *client)
{
	int ret = 0;
	struct ricoh5b_dev *ricoh5b_dev;

    #ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : Start\n", __func__);
	#endif

	if(tuner_open){
	    ricoh5b_i2c_client = client;
	    ricoh5b_dev = ricoh5b_device_info;

        ret = mt_set_gpio_out(ricoh5b_dev->vdd_gpio, 1);
        if(ret){
            pr_err("%s : ricoh5b_dev->vdd_gpio : ret= %d\n", __func__, ret);
        }
	    msleep(20);

	    ret = mt_set_gpio_out(ricoh5b_dev->ce_gpio, 1);
	    if(ret){
		    pr_err("%s : ricoh5b_dev->ce_gpio : ret= %d\n", __func__, ret);
	    }
	    msleep(20);

	    mutex_lock(&ricoh5b_dev->read_mutex);
        ret = ricoh5b_initialize_setting();
	    if(ret < 0){
            pr_err("%s:ricoh5b_initialize_setting err = %d \n",__func__, ret);
            goto send_err;
        }
	    
	    ret = ricoh5b_audio_output_mute_on();
	    if(ret < 0){
            pr_err("%s:ricoh5b_audio_output_mute_on err = %d \n",__func__, ret);
            goto send_err;
        }
        
	    ret = ricoh5b_send_audio_output_setting(RICOH5B_SELECT_AUDIO_OUTPUT);
	    if(ret < 0){
            pr_err("%s:ricoh5b_send_audio_output_setting err = %d \n",__func__, ret);
            goto send_err;
        }
	    
	    ret = ricoh5b_fm_send_set_band(ricoh5b_dev->tuned_freq / 10, gain_value);
	    if(ret < 0){
            pr_err("%s:ricoh5b_fm_send_set_band err = %d \n",__func__, ret);
            goto send_err;
        }

	    if(ricoh5b_dev->audio_mode == V4L2_TUNER_MODE_STEREO){
		    ret = ricoh5b_cmd_write_bit( RICOH5B_REG_ADR_MONO, RICOH5B_REG_DAT_MONO_OFF, RICOH5B_REG_BIT_MONO);
	    }else if(ricoh5b_dev->audio_mode == V4L2_TUNER_MODE_MONO){
		    ret = ricoh5b_cmd_write_bit( RICOH5B_REG_ADR_MONO, RICOH5B_REG_DAT_MONO_ON, RICOH5B_REG_BIT_MONO);
	    }
	    
	    if(ret < 0){
            pr_err("%s:RICOH5B_REG_ADR_MONO err = %d \n",__func__, ret);
            goto send_err;
        }

	    ret = ricoh5b_threshold_set(ricoh5b_dev, ricoh5b_dev->sense_mode);
	    if(ret < 0){
            pr_err("%s:ricoh5b_threshold_set err = %d \n",__func__, ret);
            goto send_err;
        }

	    if(ricoh5b_dev->rds_mode == RICOH5B_RDS_MODE_ON){
		    ret = ricoh5b_rds_set_status( ricoh5b_dev, RICOH5B_RDS_MODE_ON);
	    }else if(ricoh5b_dev->rds_mode == RICOH5B_RDS_MODE_OFF){
		    ret = ricoh5b_rds_set_status( ricoh5b_dev, RICOH5B_RDS_MODE_OFF);
	    }
	    if(ret < 0){
            pr_err("%s:ricoh5b RDS set err = %d \n",__func__, ret);
            goto send_err;
        }

        ret = ricoh5b_send_set_de_emphasis(ricoh5b_dev);
       	if(ret < 0){
            pr_err("%s:ricoh5b_send_set_de_emphasis err = %d \n",__func__, ret);
            goto send_err;
        }

        if(ricoh5b_dev->mute_state == RICOH5B_MUTE_OFF){
            ret = ricoh5b_audio_output_mute_off();
	        if(ret < 0){
                pr_err("%s:ricoh5b_audio_output_mute_off err = %d \n",__func__, ret);
                goto send_err;
            }else{
                ricoh5b_dev->mute_state = RICOH5B_MUTE_OFF;
            }
        }

        mutex_unlock(&ricoh5b_dev->read_mutex);
    }

	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("%s : End\n", __func__);
	#endif

	return 0;

send_err:
    mutex_unlock(&ricoh5b_dev->read_mutex);
    return ret;
}

static void ricoh5b_shutdown(struct i2c_client *client)
{
    struct ricoh5b_dev *ricoh5b_dev;
    int ret = 0;

    #ifdef RICOH5B_MAIN_DEBUG
    pr_info("%s : Start\n", __func__);
    #endif

	ricoh5b_dev = i2c_get_clientdata(client);
	
	mt_eint_mask(RICOH5B_RDSI_EINT);

	cancel_delayed_work(&ricoh5b_check_work);
    flush_workqueue(workqueue_check);
    destroy_workqueue(workqueue_check);

	cancel_delayed_work(&ricoh5b_dev->ricoh5b_rds_work);
    flush_workqueue(ricoh5b_dev->workqueue);
    destroy_workqueue(ricoh5b_dev->workqueue);

    if(tuner_open){
        /* ricoh5b off Sequence */
        ret = ricoh5b_power_stop();
        if(ret){
            pr_err("%s : ricoh5b_power_stop : ret= %d\n", __func__, ret);
        }
    }

    ret = mt_set_gpio_out(ricoh5b_dev->vdd_gpio, 0);
    if(ret){
        pr_err("%s : ricoh5b_dev->vdd_gpio : ret= %d\n", __func__, ret);
    }

    ret = mt_set_gpio_out(ricoh5b_dev->ce_gpio, 0);
    if(ret){
        pr_err("%s : ricoh5b_dev->ce_gpio : ret= %d\n", __func__, ret);
    }

    #ifdef RICOH5B_MAIN_DEBUG
    pr_info("%s : End\n", __func__);
    #endif

    return;
}

static const struct i2c_device_id ricoh5b_id[] = {
	{"ricoh5b", 0},
	{}
};

MODULE_DEVICE_TABLE(i2c, ricoh5b_id);

static struct i2c_driver ricoh5b_driver = {
	.id_table = ricoh5b_id,
	.probe    = ricoh5b_probe,
	.remove   = ricoh5b_remove,
	.suspend  = ricoh5b_suspend,
	.resume   = ricoh5b_resume,
	.shutdown = ricoh5b_shutdown,
	.driver   = {
			 .owner = THIS_MODULE,
			 .name = "ricoh5b",
			 },
};

/*
 * module load/unload record keeping
 */

static int __init ricoh5b_dev_init(void)
{
	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("Loading ricoh5b driver\n");
	#endif

	return i2c_add_driver(&ricoh5b_driver);
}

module_init(ricoh5b_dev_init);

static void __exit ricoh5b_dev_exit(void)
{
	#ifdef RICOH5B_MAIN_DEBUG
	pr_info("Unloading ricoh5b driver -S\n");
	#endif
	i2c_del_driver(&ricoh5b_driver);
}

module_exit(ricoh5b_dev_exit);

MODULE_DESCRIPTION("RICOH Tuner ricoh5b driver");
MODULE_LICENSE("GPL");
